/********************************************************************************//*!
 * @file  util_time_measurement.h 
 * @brief  Time measurement programs.
 * 
 * @details  Provides a mechanism to measure the time interval specified, using 
 *           the global timer with clock Manager.
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/13  ---  #6242  VK쐬. 
 * @date 2014/01/20  ---  -----  ُ̍CiRgA֐̃xړύX). 
 * 
 * @attention 
 * Copyright (C) 2013-2014 ALTIMA Corp./ELSENA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

#ifndef __UTIL_TIME_MEASUREMENT_H__
#define __UTIL_TIME_MEASUREMENT_H__

/***********************************************************************************
 *  includes 
 ***********************************************************************************/

/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  proto types 
 ***********************************************************************************/

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
#define PRINT_WITH_GLOBALTMR	printf("[0x%08X]", (int)alt_globaltmr_counter_get_low32());printf

#define	UTIL_TIME_MAX_TRACE			32		/*!< The maximum number of time-counter info. */

/*! Timer interval is calculated using the following equation:
 * Interval = ((PRESCALER_value + 1) * (Load_value + 1) / PERIPHCLK)
 * ## PERIPHCLK = ALT_CLK_MPU_PERIPH
 */
#define	UTIL_TIME_GLOBALTIMER_PRESCALE	1	/*!< PRESCALER_value set to 1(for MAINCLK=800MHz) or 2(for 600MHz) */
											///<       <800MHz div 4 --> 200MHz>  <600MHz div 4 --> 150MHz>
											///< - 0 --> ( 5.0nsec/count@200MHz,  6.6666..nsec/count@150MHz)
											///< - 1 --> (10.0nsec/count@200MHz, 13.3333..nsec/count@150MHz)
											///< - 2 --> (15.0nsec/count@800MHz, 19.9999..nsec/count@600MHz)
											///< - 3 --> (20.0nsec/count@800MHz)

#define	UTIL_TIME_NSEC_PER_COUNT		10	/*!< 10(10.0nsec/count@200MHz) or 20(19.9999..nsec/count@600MHz) */

/*! Measurement information record's status */
typedef enum{
	enumUTIL_TIME_STATUS_INIT,
	enumUTIL_TIME_STATUS_START,
	enumUTIL_TIME_STATUS_END,
} UtilTimeStatus_et;

/*! Measurement Points (######## This sample-specific definitions ########) */
typedef enum {
	MP_ALL,
	MP_SETUP_DMA_ALL,
	MP_SETUP_FPGA_ALL,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_RETRY1,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_RETRY2,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_RETRY3,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_RETRY4,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP1,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP2,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP3,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP4,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP5,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP6,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP7,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP8,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP9,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP10,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP11,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP12,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP13,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP14,
	MP_SETUP_FPGA_CONFIGURE_BY_DMA_DETAIL_STEP15,
	MP_SETUP_BRIDGE_ALL,
	MP_DEMOSTRATION_IO_ACROSS_BRIDGE_ALL,
	MP_CLEAN_BRIDGE_ALL,
	MP_CLEAN_FPGA_ALL,
	MP_CLEAN_DMA_ALL,
	MP_MAX
} MEASPOINT_et;

/*! Type of measurement information record. */
typedef struct{
	uint64_t			start;
	uint64_t			end;
	UtilTimeStatus_et	status;
} UtilTimeBuf_st;

/***********************************************************************************
 *  variables
 ***********************************************************************************/

/***********************************************************************************
 *  functions
 ***********************************************************************************/
extern void util_time_init(void);
extern void util_time_uninit(void);
extern void util_time_record_start_point(uint32_t index);
extern void util_time_record_end_point(uint32_t index);
extern void util_time_print_result_by_counter(uint32_t index);
extern void util_time_print_result_by_seconds(uint32_t index);
extern void util_time_print_result_all(void);
extern void util_time_print_result_all_and_clear(void);


#endif /* __UTIL_TIME_MEASUREMENT_H__ */
/***********************************************************************************
 * end of file 
 ***********************************************************************************/
