/********************************************************************************//*!
 * @file  sample_dmac.c
 * @brief  To use HWLib for DMA Controller API example program.
 * 
 * @details  Run the API category below.
 *           - DMA Controller API
 *               - DMA API for Configuration, Control, and Status
 *               - DMA API for Standard Operations
 *           - DMA Controller Programming API (Part of APIs)
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 * @data 2014/03/04  ---  #13246  SingleBurstwDMA]ǉ.
 * @date 2014/08/18  ---  -----  v14.0 (AA̕\ǉ). 
 * 
 * @attention 
 * Copyright (C) 2013-2014 ALTIMA Corp./ELSENA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

/***********************************************************************************
 *  includes 
 ***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hwlib.h"
#include "socal/alt_sysmgr.h"
#include "alt_mpu_registers.h"
#include "alt_generalpurpose_io.h"
#include "alt_globaltmr.h"
#include "alt_system_manager.h"
#include "alt_clock_manager.h"
#include "alt_interrupt.h"
#include "alt_address_space.h"
#include "alt_dma.h"
#include "alt_dma_program.h"
#include "util_time_measurement.h"

/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
// NOTE: To enable debugging output, delete the next line and uncomment the
//   line after.
#define dprintf(...)
// #define dprintf(fmt, ...) printf(fmt, ##__VA_ARGS__)

/** memo: port asign ***************/
/* Signal Name <--> HPS Port Name  */
/* USER_DIPSW_HPS0 <--> HPS_GPI13  */
/* USER_DIPSW_HPS1 <--> HPS_GPI12  */
/* USER_DIPSW_HPS2 <--> HPS_GPI11  */
/* USER_DIPSW_HPS3 <--> HPS_GPI10  */
/* USER_PB_HPS0 <--> HPS_GPI9      */
/* USER_PB_HPS1 <--> HPS_GPI8      */
/* USER_PB_HPS2 <--> HPS_GPI7      */
/* USER_PB_HPS3 <--> HPS_GPI6      */
/* USER_HPS_GPIO61 <--> HPS_GPIO61 */
/***********************************/
#define ALT_GPIO_PORTBIT_DIPS1		(ALT_GPIO_BIT26)
#define ALT_GPIO_PORTBIT_DIPS2		(ALT_GPIO_BIT25)
#define ALT_GPIO_PORTBIT_DIPS3		(ALT_GPIO_BIT24)
#define ALT_GPIO_PORTBIT_DIPS4		(ALT_GPIO_BIT23)
#define ALT_GPIO_PORTBIT_DIPSWALL	((ALT_GPIO_BIT26)|(ALT_GPIO_BIT25)|(ALT_GPIO_BIT24)|(ALT_GPIO_BIT23))
#define ALT_GPIO_LSBBIT_DIPSWS		(23)
#define ALT_GPIO_PORTBIT_PUSH2		(ALT_GPIO_BIT22)
#define ALT_GPIO_PORTBIT_PUSH0		(ALT_GPIO_BIT21)
#define ALT_GPIO_PORTBIT_PUSH3		(ALT_GPIO_BIT20)
#define ALT_GPIO_PORTBIT_PUSH1		(ALT_GPIO_BIT19)
#define ALT_GPIO_PORTBIT_PUSHSWALL	((ALT_GPIO_BIT22)|(ALT_GPIO_BIT21)|(ALT_GPIO_BIT20)|(ALT_GPIO_BIT19))
#define ALT_GPIO_PORTBIT_SWALL		((ALT_GPIO_PORTBIT_DIPSWALL)|(ALT_GPIO_PORTBIT_PUSHSWALL))

typedef void (*CALLBACK_FUNKt)(uint32_t, void*);

typedef enum {
	DMATYPEe_NON = -1,
	DMATYPEe_M2M,
	DMATYPEe_Z2M,
	DMATYPEe_M2R,
	DMATYPEe_R2M,
	DMATYPEe_M2P,
	DMATYPEe_P2M,
	DMATYPEe_M2MS,
	DMATYPEe_M2RS,
	DMATYPEe_R2MS,
	DMATYPEe_R2RS
} DMATYPEe;

typedef struct {
	DMATYPEe type;
    void*	dst_address;
    void*	src_address;
    size_t	trans_size;
    uint32_t width_bits;	// Valid values are 8, 16, 32, and 64.
} DMA_TEST_SETTINGt;

/***********************************************************************************
 *  proto types
 ***********************************************************************************/
static void sample_dmac_setting_remap(void);
static void sample_dmac_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end);
static void sample_sysmgr_print_all_fpga_interface(void);
static void sample_sysmgr_enable_all_fpga_interface(void);
static void sample_sysmgr_disable_all_fpga_interface(void);
static void sample_dmac_print_manager_status(void);
static void sample_dmac_print_ch_status(ALT_DMA_CHANNEL_t channel, bool detail);
static void sample_dmac_print_all_status(void);
static void sample_dmac_transfer_start(ALT_DMA_CHANNEL_t channel, ALT_DMA_EVENT_t event, DMA_TEST_SETTINGt* test_p, bool restart);
void sample_dmac_callback_DMA_IRQ0(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ1(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ2(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ3(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ4(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ5(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ6(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ7(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_IRQ_ABORT(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_ECC_CORRECTED_IRQ(uint32_t icciar, void *context);
void sample_dmac_callback_DMA_ECC_UNCORRECTED_IRQ(uint32_t icciar, void *context);
void sample_dmac_setting_gic(void);
ALT_STATUS_CODE sample_dmac_single_transfer(ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t* program,
		void* dst_address, const void* src_address, size_t count, uint32_t width_bits,
		bool dst_inc, bool src_inc, bool send_evt, ALT_DMA_EVENT_t evt);
void sample_test_init(void);
void sample_test_uninit(void);
void sample_test_print_usage(void);
void sample_test_main(void);
// prototype (Common Functions) //
static void sample_gpio_config_helioboard(void);

/***********************************************************************************
 *  variables 
 ***********************************************************************************/
ALT_DMA_CFG_t dma_config;			/*!< DMA Configration Table. */
ALT_DMA_PROGRAM_t dma_program[8];	/*!< Micro code program buffer for the DMAC (The index# of the array means the DMA channel#) */
/*! DMA Test Setting Parameters */
DMA_TEST_SETTINGt dma_setting[8] = {
	/*	type,	dst_address,	src_address,	trans_size, 	width_bits*/
	{DMATYPEe_M2M, (void*)0x04000000, (void*)0x05000000, (2 * 0x100000), 32	},	/* trans_size=2MiB (## Error occurs when you specify more than 2 MiB size.) */
	{DMATYPEe_Z2M, (void*)0x04000000, (void*)0x05000000, (3 * 0x100000), 32	},	/* trans_size=3MiB (## Error occurs when you specify more than 3 MiB size.) */
	{DMATYPEe_M2R, (void*)0x04000000, (void*)0x05000000, (3 * 0x100000), 64	},	/* trans_size=3MiB (## Error occurs when you specify more than 3 MiB size.) */
	{DMATYPEe_R2M, (void*)0x04000000, (void*)0x05000000, (3 * 0x100000), 64	},	/* trans_size=3MiB (## Error occurs when you specify more than 3 MiB size.) */
	{DMATYPEe_M2M, (void*)0x04800000, (void*)0x05800000, (2 * 0x100000), 32	},	/* trans_size=2MiB (## Error occurs when you specify more than 2 MiB size.) */
	{DMATYPEe_Z2M, (void*)0x04800000, (void*)0x05800000, (3 * 0x100000), 32	},	/* trans_size=3MiB (## Error occurs when you specify more than 3 MiB size.) */
	{DMATYPEe_M2R, (void*)0x04800000, (void*)0x05800000, (3 * 0x100000), 64	},	/* trans_size=3MiB (## Error occurs when you specify more than 3 MiB size.) */
	{DMATYPEe_R2M, (void*)0x04800000, (void*)0x05800000, (3 * 0x100000), 64	}	/* trans_size=3MiB (## Error occurs when you specify more than 3 MiB size.) */
};

/*! Status String Definitions of DMA Manager */
//typedef enum ALT_DMA_MANAGER_STATE_e
//{
//    ALT_DMA_MANAGER_STATE_STOPPED     = 0, /*!< Stopped */
//    ALT_DMA_MANAGER_STATE_EXECUTING   = 1, /*!< Executing */
//    ALT_DMA_MANAGER_STATE_CACHE_MISS  = 2, /*!< Cache Miss */
//    ALT_DMA_MANAGER_STATE_UPDATING_PC = 3, /*!< Updating PC */
//    ALT_DMA_MANAGER_STATE_WFE         = 4, /*!< Waiting for Event */
//    ALT_DMA_MANAGER_STATE_FAULTING    = 15 /*!< Faulting */
//}
//ALT_DMA_MANAGER_STATE_t;
const char* StatusStringDMAManager[16] = {
	"STOPPED(0)",
	"EXECUTING(1)",
	"CACHE_MISS(2)",
	"UPDATING_PC(3)",
	"WFE(4)",
	"---(5)",
	"---(6)",
	"---(7)",
	"---(8)",
	"---(9)",
	"---(10)",
	"---(11)",
	"---(12)",
	"---(13)",
	"---(14)",
	"FAULTING(15)"
};
/*! Status String Definitions of DMA Channel */
//typedef enum ALT_DMA_CHANNEL_STATE_e
//{
//    ALT_DMA_CHANNEL_STATE_STOPPED             = 0,  /*!< Stopped */
//    ALT_DMA_CHANNEL_STATE_EXECUTING           = 1,  /*!< Executing */
//    ALT_DMA_CHANNEL_STATE_CACHE_MISS          = 2,  /*!< Cache Miss */
//    ALT_DMA_CHANNEL_STATE_UPDATING_PC         = 3,  /*!< Updating PC */
//    ALT_DMA_CHANNEL_STATE_WFE                 = 4,  /*!< Waiting for Event */
//    ALT_DMA_CHANNEL_STATE_AT_BARRIER          = 5,  /*!< At Barrier */
//    ALT_DMA_CHANNEL_STATE_WFP                 = 7,  /*!< Waiting for Peripheral */
//    ALT_DMA_CHANNEL_STATE_KILLING             = 8,  /*!< Killing */
//    ALT_DMA_CHANNEL_STATE_COMPLETING          = 9,  /*!< Completing */
//    ALT_DMA_CHANNEL_STATE_FAULTING_COMPLETING = 14, /*!< Faulting Completing */
//    ALT_DMA_CHANNEL_STATE_FAULTING            = 15  /*!< Faulting */
//}
//ALT_DMA_CHANNEL_STATE_t;
const char* StatusStringDMAChannel[16] = {
	"STOPPED(0)",
	"EXECUTING(1)",
	"CACHE_MISS(2)",
	"UPDATING_PC(3)",
	"WFE(4)",
	"AT_BARRIER(5)",
	"---(6)",
	"WFP(7)",
	"KILLING(8)",
	"COMPLETING(9)",
	"---(10)",
	"---(11)",
	"---(12)",
	"---(13)",
	"FAULTING_COMPLETING(14)",
	"FAULTING(15)"
};

/*! Interrupt Callback Function Table */
const CALLBACK_FUNKt	SAMPLE_CallBacks[11] =
{
		sample_dmac_callback_DMA_IRQ0,
		sample_dmac_callback_DMA_IRQ1,
		sample_dmac_callback_DMA_IRQ2,
		sample_dmac_callback_DMA_IRQ3,
		sample_dmac_callback_DMA_IRQ4,
		sample_dmac_callback_DMA_IRQ5,
		sample_dmac_callback_DMA_IRQ6,
		sample_dmac_callback_DMA_IRQ7,
		sample_dmac_callback_DMA_IRQ_ABORT,
		sample_dmac_callback_DMA_ECC_CORRECTED_IRQ,
		sample_dmac_callback_DMA_ECC_UNCORRECTED_IRQ
};

int	SAMPLE_CallBackCount[11];	/*!< Interrupt detection counters */

/***********************************************************************************
 *  functions 
 ***********************************************************************************/

/********************************************************************************//*!
 * @fn  static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext)
 * @brief  Writes the data for transfer test.
 *
 * @details  Writes the address value area of memory that is specified for the transfer test.
 * @param[in] start  start address
 * @param[in] endnext  Next to the end address
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_dmac_setting_testdata(uint32_t* start, uint32_t* endnext)
{
	while(start < endnext){
		*start = (uint32_t)start;
		start++;
	}

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_dmac_setting_remap(void)
 * @brief  Setting for the Address Space Remap Register.
 *
 * @details  Run the following APIs.
 *           - alt_addr_space_remap
 *           - alt_mpu_addr_space_remap_0_to_sdram
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_dmac_setting_remap(void)
{
	ALT_STATUS_CODE result_code;
	// Remap Setting Value 
	ALT_ADDR_SPACE_MPU_ATTR_t			mpu_attr = ALT_ADDR_SPACE_MPU_ZERO_AT_OCRAM;
	ALT_ADDR_SPACE_NONMPU_ATTR_t		nonmpu_attr = ALT_ADDR_SPACE_NONMPU_ZERO_AT_SDRAM;
	ALT_ADDR_SPACE_H2F_BRIDGE_ATTR_t	h2f_attr = ALT_ADDR_SPACE_H2F_ACCESSIBLE;
	ALT_ADDR_SPACE_LWH2F_BRIDGE_ATTR_t	lwh2f_attr = ALT_ADDR_SPACE_LWH2F_ACCESSIBLE;
	
	/* -- Address Space Mapping Control -- */
	/* Usage:	ALT_STATUS_CODE  alt_addr_space_remap (ALT_ADDR_SPACE_MPU_ATTR_t mpu_attr, ALT_ADDR_SPACE_NONMPU_ATTR_t nonmpu_attr, ALT_ADDR_SPACE_H2F_BRIDGE_ATTR_t h2f_attr, ALT_ADDR_SPACE_LWH2F_BRIDGE_ATTR_t lwh2f_attr);	*/
	/* Usage:	ALT_STATUS_CODE  alt_mpu_addr_space_remap_0_to_sdram (void);	*/
	printf("==== HWLib test : Execute the Address Space Remap Setting ====\n");
	
	result_code = alt_addr_space_remap(mpu_attr, nonmpu_attr, h2f_attr, lwh2f_attr);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_addr_space_remap() .... result=%d\n", (int)result_code);
	
	result_code = alt_mpu_addr_space_remap_0_to_sdram();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_mpu_addr_space_remap_0_to_sdram() .... result=%d\n", (int)result_code);
	
	printf("\n");
	
	return;
}

/********************************************************************************//*!
 * @fn  static void sample_dmac_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end)
 * @brief  Setting for the L2 Cache Address Filter.
 *
 * @details  Run the following APIs.
 *           - alt_l2_addr_filter_cfg_get
 *           - alt_l2_addr_filter_cfg_set
 * @param[in] addr_filt_start  Filter Start Address.
 * @param[in] addr_filt_end  Filter End Address.
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_dmac_setting_l2addrfilter(uint32_t addr_filt_start, uint32_t addr_filt_end)
{
	ALT_STATUS_CODE result_code;
	uint32_t	before_filt_start;
	uint32_t	before_filt_end;
	
	/* -- L2 Cache Address Filter -- */
	/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_get (uint32_t *addr_filt_start, uint32_t *addr_filt_end);	*/
	/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_set (uint32_t addr_filt_start, uint32_t addr_filt_end);	*/
	printf("==== HWLib test : Execute the L2 Cache Address Filter Setting ====\n");
	result_code = alt_l2_addr_filter_cfg_get(&before_filt_start, &before_filt_end);
	printf("<<before setting>>\n");
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_l2_addr_filter_cfg_get() .... result=%d\n", (int)result_code);
	printf(" +-- addr_filt_start = 0x%08X\n", (int)before_filt_start);
	printf(" +-- addr_filt_end   = 0x%08X\n", (int)before_filt_end);
	
	result_code = alt_l2_addr_filter_cfg_set(addr_filt_start, addr_filt_end);
	printf("<<after setting>>\n");
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_l2_addr_filter_cfg_set() .... result=%d\n", (int)result_code);
	printf(" +-- addr_filt_start = 0x%08X\n", (int)addr_filt_start);
	printf(" +-- addr_filt_end   = 0x%08X\n", (int)addr_filt_end);
	
	printf("\n");
	
	return;
}

/********************************************************************************//*!
 * @fn  static void sample_sysmgr_print_all_fpga_interface(void)
 * @brief  Printing all FPGA interface enable State.
 *
 * @details  Run the following API for all FPGA interface.
 *           - alt_fpga_interface_is_enabled
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_sysmgr_print_all_fpga_interface(void)
{
	/* Usage:	ALT_STATUS_CODE alt_fpga_interface_is_enabled  ( ALT_FPGA_INTERFACE_t  intfc );	*/
	printf("ALT_FPGA_INTERFACE_GLOBAL ...... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_GLOBAL));
	printf("ALT_FPGA_INTERFACE_RESET_REQ ... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_RESET_REQ));
	printf("ALT_FPGA_INTERFACE_JTAG_ENABLE . %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_JTAG_ENABLE));
	printf("ALT_FPGA_INTERFACE_CONFIG_IO ... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_CONFIG_IO));
	printf("ALT_FPGA_INTERFACE_BSCAN ....... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_BSCAN));
	printf("ALT_FPGA_INTERFACE_TRACE ....... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_TRACE));
	printf("ALT_FPGA_INTERFACE_DBG_APB ..... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_DBG_APB));
	printf("ALT_FPGA_INTERFACE_STM ......... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_STM));
	printf("ALT_FPGA_INTERFACE_CTI ......... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_CTI));
	printf("ALT_FPGA_INTERFACE_EMAC0 ....... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_EMAC0));
	printf("ALT_FPGA_INTERFACE_EMAC1 ....... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_EMAC1));
	printf("ALT_FPGA_INTERFACE_SPIM0 ....... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_SPIM0));
	printf("ALT_FPGA_INTERFACE_SPIM1 ....... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_SPIM1));
	printf("ALT_FPGA_INTERFACE_NAND ........ %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_NAND));
	printf("ALT_FPGA_INTERFACE_SDMMC ....... %d\n", (int)alt_fpga_interface_is_enabled(ALT_FPGA_INTERFACE_SDMMC));

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_sysmgr_enable_all_fpga_interface(void)
 * @brief  Setting all FPGA interface enable State to Enabled.
 *
 * @details  Run the following API for all FPGA interface.
 *           - alt_fpga_interface_enable
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_sysmgr_enable_all_fpga_interface(void)
{
	ALT_STATUS_CODE result_code;

	/* Usage:	ALT_STATUS_CODE alt_fpga_interface_enable  ( ALT_FPGA_INTERFACE_t  intfc ):	 */
	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_GLOBAL);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_GLOBAL) .... result=%d\n", (int)result_code);

#if 0
	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_RESET_REQ);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_RESET_REQ) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_JTAG_ENABLE);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_JTAG_ENABLE) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_CONFIG_IO);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_CONFIG_IO) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_BSCAN);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_BSCAN) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_TRACE);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_TRACE) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_DBG_APB);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_DBG_APB) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_STM);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_STM) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_CTI);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_CTI) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_EMAC0);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_EMAC0) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_EMAC1);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_EMAC1) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_SPIM0);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_SPIM0) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_SPIM1);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_SPIM1) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_NAND);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_NAND) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_enable(ALT_FPGA_INTERFACE_SDMMC);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_enable(ALT_FPGA_INTERFACE_SDMMC) .... result=%d\n", (int)result_code);
#endif

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_sysmgr_disable_all_fpga_interface(void)
 * @brief  Setting all FPGA interface enable State to Disabled.
 *
 * @details  Run the following API for all FPGA interface.
 *           - alt_fpga_interface_disable
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_sysmgr_disable_all_fpga_interface(void)
{
	ALT_STATUS_CODE result_code;

	/* Usage:	ALT_STATUS_CODE alt_fpga_interface_disable  ( ALT_FPGA_INTERFACE_t  intfc ):	 */
	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_GLOBAL);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_GLOBAL) .... result=%d\n", (int)result_code);

#if 0
	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_RESET_REQ);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_RESET_REQ) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_JTAG_ENABLE);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_JTAG_ENABLE) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_CONFIG_IO);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_CONFIG_IO) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_BSCAN);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_BSCAN) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_TRACE);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_TRACE) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_DBG_APB);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_DBG_APB) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_STM);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_STM) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_CTI);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_CTI) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_EMAC0);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_EMAC0) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_EMAC1);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_EMAC1) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_SPIM0);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_SPIM0) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_SPIM1);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_SPIM1) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_NAND);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_NAND) .... result=%d\n", (int)result_code);

	result_code = alt_fpga_interface_disable(ALT_FPGA_INTERFACE_SDMMC);
	if(result_code != ALT_E_SUCCESS){
		//printf("ERROR!!: ");
	}
	printf("alt_fpga_interface_disable(ALT_FPGA_INTERFACE_SDMMC) .... result=%d\n", (int)result_code);
#endif

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_dmac_print_manager_status(void)
 * @brief  Print the DMA Manager status and information.
 *
 * @details  Run the following APIs.
 *           - alt_dma_manager_state_get
 *           - alt_dma_manager_fault_status_get
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_dmac_print_manager_status(void)
{
	ALT_STATUS_CODE result_code;
	ALT_DMA_MANAGER_STATE_t dma_manager_state;
	ALT_DMA_MANAGER_FAULT_t dma_manager_fault_state;
	const char*	status_string_p;

	printf("==== Print DMA Manager Status ====\n");
	// -- DMA Manager Status --
	/* Usage:	ALT_STATUS_CODE  alt_dma_manager_state_get (ALT_DMA_MANAGER_STATE_t *state);	*/
	result_code = alt_dma_manager_state_get(&dma_manager_state);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	if((dma_manager_state < ALT_DMA_MANAGER_STATE_STOPPED)||(ALT_DMA_MANAGER_STATE_FAULTING < dma_manager_state)){
		status_string_p = NULL;
	} else {
		status_string_p = StatusStringDMAManager[dma_manager_state];
	}
	printf("alt_dma_manager_state_get()       : %s .... result=%d\n", status_string_p, (int)result_code);
	if(dma_manager_state == ALT_DMA_MANAGER_STATE_FAULTING){
		// -- DMA Manager Fault Status --
		/* Usage:	ALT_STATUS_CODE  alt_dma_manager_fault_status_get (ALT_DMA_MANAGER_FAULT_t *fault);	*/
		result_code = alt_dma_manager_fault_status_get(&dma_manager_fault_state);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_dma_manager_fault_status_get(): 0x%08X .... result=%d\n", (int)dma_manager_fault_state, (int)result_code);
	}

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_dmac_print_ch_status(ALT_DMA_CHANNEL_t channel, bool detail)
 * @brief  Print the DMA Channel status and information.
 *
 * @details  Run the following APIs for specified DMA CH.
 *           - alt_dma_channel_state_get
 *           - alt_dma_channel_fault_status_get
 *           - alt_dma_event_int_status_get_raw
 *           - alt_dma_int_status_get
 *           - alt_dma_channel_reg_get
 *           - alt_dma_program_progress_reg
 * @param[in] channel  target DMA CH.
 * @param[in] detail  To display more information if true.
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_dmac_print_ch_status(ALT_DMA_CHANNEL_t channel, bool detail)
{
	ALT_STATUS_CODE result_code;
	ALT_DMA_CHANNEL_STATE_t dma_channel_state;
	ALT_DMA_CHANNEL_FAULT_t dma_channel_fault_state;
	ALT_DMA_EVENT_t evt_num;
	uint32_t reg_DMA_CCR;
	uint32_t reg_DMA_SAR;
	uint32_t reg_DMA_DAR;
	uint32_t progress;
	uint32_t trans_size;
	const char*	status_string_p;

	printf("==== Print DMA Status [CH:%d] ====\n", (int)channel);
	// -- DMA CH Resources Status --
	/* Usage:	ALT_STATUS_CODE  alt_dma_channel_state_get (ALT_DMA_CHANNEL_t channel, ALT_DMA_CHANNEL_STATE_t *state);	*/
	result_code = alt_dma_channel_state_get(channel, &dma_channel_state);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	if((dma_channel_state < ALT_DMA_CHANNEL_STATE_STOPPED)||(ALT_DMA_CHANNEL_STATE_FAULTING < dma_channel_state)){
		status_string_p = NULL;
	} else {
		status_string_p = StatusStringDMAChannel[dma_channel_state];
	}
	printf("alt_dma_channel_state_get()       : %s .... result=%d\n", status_string_p, (int)result_code);
	if((dma_channel_state == ALT_DMA_CHANNEL_STATE_FAULTING_COMPLETING)||(dma_channel_state == ALT_DMA_CHANNEL_STATE_FAULTING)){
		// -- DMA CH Falut Status --
		/* Usage:	ALT_STATUS_CODE  alt_dma_channel_fault_status_get (ALT_DMA_CHANNEL_t channel, ALT_DMA_CHANNEL_FAULT_t *fault);	*/
		result_code = alt_dma_channel_fault_status_get(channel, &dma_channel_fault_state);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_dma_channel_fault_status_get(): 0x%08X .... result=%d\n", (int)dma_channel_fault_state, (int)result_code);
	}

	if(detail){
		evt_num = (ALT_DMA_EVENT_t)(ALT_DMA_EVENT_0 + channel);
		// -- DMA CH event or int status --
		/* Usage:	ALT_STATUS_CODE  alt_dma_event_int_status_get_raw (ALT_DMA_EVENT_t evt_num);	*/
		printf("alt_dma_event_int_status_get_raw(): .... result=%d\n", (int)alt_dma_event_int_status_get_raw(evt_num));
		// -- DMA CH int status --
		/* Usage:	ALT_STATUS_CODE  alt_dma_int_status_get (ALT_DMA_EVENT_t irq_num);	*/
		printf("alt_dma_int_status_get()          : .... result=%d\n", (int)alt_dma_int_status_get(evt_num));
	}

	/* Usage:	ALT_STATUS_CODE  alt_dma_channel_reg_get (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_REG_t reg, uint32_t *val);	*/
	result_code = alt_dma_channel_reg_get(channel, ALT_DMA_PROGRAM_REG_CCR, &reg_DMA_CCR);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_dma_channel_reg_get(CCR)      : 0x%08X .... result=%d\n", (int)reg_DMA_CCR, (int)result_code);
	result_code = alt_dma_channel_reg_get(channel, ALT_DMA_PROGRAM_REG_SAR, &reg_DMA_SAR);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_dma_channel_reg_get(SAR)      : 0x%08X .... result=%d\n", (int)reg_DMA_SAR, (int)result_code);
	result_code = alt_dma_channel_reg_get(channel, ALT_DMA_PROGRAM_REG_DAR, &reg_DMA_DAR);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_dma_channel_reg_get(DAR)      : 0x%08X .... result=%d\n", (int)reg_DMA_DAR, (int)result_code);

	if(detail){
		trans_size = (uint32_t)(dma_setting[channel].trans_size);
		/* Usage:	ALT_STATUS_CODE  alt_dma_program_progress_reg (ALT_DMA_PROGRAM_t *pgm, ALT_DMA_PROGRAM_REG_t reg, uint32_t current, uint32_t *progress);	*/
		if(dma_setting[channel].type != DMATYPEe_Z2M){
			result_code = alt_dma_program_progress_reg(&dma_program[channel], ALT_DMA_PROGRAM_REG_SAR, reg_DMA_SAR, &progress);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			}
			printf("alt_dma_program_progress_reg(SAR) : 0x%08X (%3d%%) .... result=%d\n", (int)progress, (int)(progress/trans_size*100), (int)result_code);
		} else {
			printf("Progress of the SAR is not allowed to run in the channel is set to DMATYPEe_Z2M.\n");
		}
		result_code = alt_dma_program_progress_reg(&dma_program[channel], ALT_DMA_PROGRAM_REG_DAR, reg_DMA_DAR, &progress);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_dma_program_progress_reg(DAR) : 0x%08X (%3d%%) .... result=%d\n", (int)progress, (int)(progress/trans_size*100), (int)result_code);
	}

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_dmac_print_all_status(void)
 * @brief  Print all DMA status and informations.
 *
 * @details  Run the following Functions.
 *           - sample_dmac_print_manager_status
 *           - sample_dmac_print_ch_status (for all DMA CHs)
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_dmac_print_all_status(void)
{
	int	i;

	printf("======== Print All DMA Status ========\n");
	// -- DMA Manager Status --
	sample_dmac_print_manager_status();
	// -- DMA CH Resources Status --
	for(i=0; i<8; i++){
		// -- DMA CH Status --
		sample_dmac_print_ch_status((ALT_DMA_CHANNEL_t)i, false);
	}

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_dmac_transfer_start(ALT_DMA_CHANNEL_t channel, ALT_DMA_EVENT_t event, DMA_TEST_SETTINGt* test_p, bool restart)
 * @brief  Start the DMA at CH specified with parameter.
 *
 * @details  Start the DMA transfer in the following conditions.
 *           - restart is false
 *             After generating a DMA CH parameter and see DMAC Micro Code to run the DMA transfer.
 *           - restart is true
 *             Restart the DMA transfers using DMA CH parameter has already been generated.
 * @param[in] channel  target DMA CH.
 * @param[in] evnet  Event number to notify when the transfer completes.
 * @param[in] test_p  Pointer to a DMA test parameters (info address, size, etc.).
 * @param[in] restart  Specify true to start the transfer by a restart.
 * @retval void  none
 *
 * @attention  When you first start calling this function in the restart = true. for each channels.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
static void sample_dmac_transfer_start(ALT_DMA_CHANNEL_t channel, ALT_DMA_EVENT_t event, DMA_TEST_SETTINGt* test_p, bool restart)
{
	ALT_STATUS_CODE result_code;
	int topidx = channel * 4;
	uint32_t* dstp;
	uint32_t* srcp;
	bool dst_inc = true;
	bool src_inc = true;
	
	switch(test_p->type)
	{
	case DMATYPEe_M2M:	// --------------------------------------------------- 
		// Setting TEST Data!!! //
		sample_dmac_setting_testdata((uint32_t*)test_p->dst_address, (uint32_t*)(test_p->dst_address + test_p->trans_size));
		sample_dmac_setting_testdata((uint32_t*)test_p->src_address, (uint32_t*)(test_p->src_address + test_p->trans_size));
		
		if(!restart){	/* ---- restart is false ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc (ALT_DMA_CHANNEL_t channel);	*/
			util_time_record_start_point(topidx + 0);
			result_code = alt_dma_channel_alloc(channel);
			util_time_record_end_point(topidx + 0);
			printf("alt_dma_channel_alloc(%d) .... result=%d\n", (int)channel, (int)result_code);
			
			/* Usage:	ALT_STATUS_CODE  alt_dma_memory_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program,
			 * 									void *dest, const void *src, size_t size, bool send_evt, ALT_DMA_EVENT_t evt);	*/
			util_time_record_start_point(topidx + 1);
			__asm("CPSID i");
			result_code = alt_dma_memory_to_memory(channel, &dma_program[channel],
					test_p->dst_address, test_p->src_address, test_p->trans_size, true, event);
			util_time_record_end_point(topidx + 1);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_memory_to_memory() .... result=%d\n", (int)result_code);
			printf("+-- CH=%d DST=0x%08X SRC=0x%08X SIZE=0x%08X\n", (int)channel, (int)test_p->dst_address, (int)test_p->src_address, (int)test_p->trans_size);
		} else {	/* ---- restart is true ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_exec (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *pgm);	*/
			util_time_record_start_point(topidx + 2);
			__asm("CPSID i");
			result_code = alt_dma_channel_exec(channel, &dma_program[channel]);
			util_time_record_end_point(topidx + 2);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_channel_exec() .... result=%d\n", (int)result_code);
		}
		break;
	case DMATYPEe_Z2M:	// --------------------------------------------------- 
		// Setting TEST Data!!! //
		sample_dmac_setting_testdata((uint32_t*)test_p->dst_address, (uint32_t*)(test_p->dst_address + test_p->trans_size));
		sample_dmac_setting_testdata((uint32_t*)test_p->src_address, (uint32_t*)(test_p->src_address + test_p->trans_size));
		
		if(!restart){	/* ---- restart is false ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc (ALT_DMA_CHANNEL_t channel);	*/
			util_time_record_start_point(topidx + 0);
			result_code = alt_dma_channel_alloc(channel);
			util_time_record_end_point(topidx + 0);
			printf("alt_dma_channel_alloc(%d) .... result=%d\n", (int)channel, (int)result_code);
			
			/* Usage:	ALT_STATUS_CODE  alt_dma_zero_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program,
			 * 									void *buf, size_t size, bool send_evt, ALT_DMA_EVENT_t evt);	*/
			util_time_record_start_point(topidx + 1);
			__asm("CPSID i");
			result_code = alt_dma_zero_to_memory(channel, &dma_program[channel],
					test_p->dst_address, test_p->trans_size, true, event);
			util_time_record_end_point(topidx + 1);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_zero_to_memory() .... result=%d\n", (int)result_code);
			printf("+-- CH=%d DST=0x%08X SRC=(zero) SIZE=0x%08X\n", (int)channel, (int)test_p->dst_address, (int)test_p->trans_size);
		} else {	/* ---- restart is true ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_exec (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *pgm);	*/
			util_time_record_start_point(topidx + 2);
			__asm("CPSID i");
			result_code = alt_dma_channel_exec(channel, &dma_program[channel]);
			util_time_record_end_point(topidx + 2);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_channel_exec() .... result=%d\n", (int)result_code);
		}
		break;
	case DMATYPEe_M2R:	// --------------------------------------------------- 
		// Setting TEST Data!!! //
		sample_dmac_setting_testdata((uint32_t*)test_p->dst_address, (uint32_t*)(test_p->dst_address + test_p->trans_size));
		sample_dmac_setting_testdata((uint32_t*)test_p->src_address, (uint32_t*)(test_p->src_address + test_p->trans_size));
		dstp = (uint32_t*)test_p->dst_address;
		dstp[0] = 0;
		dstp[1] = 0;
		srcp = (uint32_t*)(test_p->src_address + test_p->trans_size - 8);
		srcp[0] = 0x01234567;
		srcp[1] = 0x89ABCDEF;
		
		if(!restart){	/* ---- restart is false ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc (ALT_DMA_CHANNEL_t channel);	*/
			util_time_record_start_point(topidx + 0);
			result_code = alt_dma_channel_alloc(channel);
			util_time_record_end_point(topidx + 0);
			printf("alt_dma_channel_alloc(%d) .... result=%d\n", (int)channel, (int)result_code);
			
			/* Usage:	ALT_STATUS_CODE  alt_dma_memory_to_register (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *dst_reg,
			 * 					const void *src_buf, size_t count, uint32_t register_width_bits, bool send_evt, ALT_DMA_EVENT_t evt);	*/
			util_time_record_start_point(topidx + 1);
			__asm("CPSID i");
			result_code = alt_dma_memory_to_register(channel, &dma_program[channel], test_p->dst_address,
					test_p->src_address, (test_p->trans_size/(test_p->width_bits>>3)), test_p->width_bits, true, event);
			util_time_record_end_point(topidx + 1);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_memory_to_register() .... result=%d\n", (int)result_code);
			printf("+-- CH=%d DST=0x%08X SRC=0x%08X SIZE=0x%08X WIDTH=%d\n", (int)channel, (int)test_p->dst_address, (int)test_p->src_address, (int)test_p->trans_size, (int)test_p->width_bits);
		} else {	/* ---- restart is true ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_exec (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *pgm);	*/
			util_time_record_start_point(topidx + 2);
			__asm("CPSID i");
			result_code = alt_dma_channel_exec(channel, &dma_program[channel]);
			util_time_record_end_point(topidx + 2);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_channel_exec() .... result=%d\n", (int)result_code);
		}
		break;
	case DMATYPEe_R2M:	// --------------------------------------------------- 
		// Setting TEST Data!!! //
		sample_dmac_setting_testdata((uint32_t*)test_p->dst_address, (uint32_t*)(test_p->dst_address + test_p->trans_size));
		sample_dmac_setting_testdata((uint32_t*)test_p->src_address, (uint32_t*)(test_p->src_address + test_p->trans_size));
		srcp = (uint32_t*)test_p->src_address;
		srcp[0] = 0x01234567;
		srcp[1] = 0x89ABCDEF;

		if(!restart){	/* ---- restart is false ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc (ALT_DMA_CHANNEL_t channel);	*/
			util_time_record_start_point(topidx + 0);
			result_code = alt_dma_channel_alloc(channel);
			util_time_record_end_point(topidx + 0);
			printf("alt_dma_channel_alloc(%d) .... result=%d\n", (int)channel, (int)result_code);
			
			/* Usage:	ALT_STATUS_CODE  alt_dma_register_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *dst_buf,
			 * 					const void *src_reg, size_t count, uint32_t register_width_bits, bool send_evt, ALT_DMA_EVENT_t evt);	*/
			util_time_record_start_point(topidx + 1);
			__asm("CPSID i");
			result_code = alt_dma_register_to_memory(channel, &dma_program[channel], test_p->dst_address,
					test_p->src_address, (test_p->trans_size/(test_p->width_bits>>3)), test_p->width_bits, true, event);
			util_time_record_end_point(topidx + 1);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_register_to_memory() .... result=%d\n", (int)result_code);
			printf("+-- CH=%d DST=0x%08X SRC=0x%08X SIZE=0x%08X WIDTH=%d\n", (int)channel, (int)test_p->dst_address, (int)test_p->src_address, (int)test_p->trans_size, (int)test_p->width_bits);
		} else {	/* ---- restart is true ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_exec (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *pgm);	*/
			util_time_record_start_point(topidx + 2);
			__asm("CPSID i");
			result_code = alt_dma_channel_exec(channel, &dma_program[channel]);
			util_time_record_end_point(topidx + 2);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_channel_exec() .... result=%d\n", (int)result_code);
		}
		break;
	case DMATYPEe_M2P:	// --------------------------------------------------- 
	case DMATYPEe_P2M:	// --------------------------------------------------- 
		/* NOP : In this sample does not support types. */
		/* Usage:	ALT_STATUS_CODE  alt_dma_memory_to_periph (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, ALT_DMA_PERIPH_t dest, const void *src, size_t size, void *periph_info, bool send_evt, ALT_DMA_EVENT_t evt);	*/
		/* Usage:	ALT_STATUS_CODE  alt_dma_periph_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *dest, ALT_DMA_PERIPH_t src, size_t size, void *periph_info, bool send_evt, ALT_DMA_EVENT_t evt);	*/
		break;
	case DMATYPEe_M2MS:	// --------------------------------------------------- 
	case DMATYPEe_M2RS:	// --------------------------------------------------- 
	case DMATYPEe_R2MS:	// --------------------------------------------------- 
	case DMATYPEe_R2RS:	// --------------------------------------------------- 
		// Setting TEST Data!!! //
		sample_dmac_setting_testdata((uint32_t*)test_p->dst_address, (uint32_t*)(test_p->dst_address + test_p->trans_size));
		sample_dmac_setting_testdata((uint32_t*)test_p->src_address, (uint32_t*)(test_p->src_address + test_p->trans_size));
		
		if((test_p->type == DMATYPEe_M2RS)||(test_p->type == DMATYPEe_R2RS)){
			dst_inc = false;
		}
		if((test_p->type == DMATYPEe_R2MS)||(test_p->type == DMATYPEe_R2RS)){
			src_inc = false;
			srcp = (uint32_t*)test_p->src_address;
			srcp[0] = 0x01234567;
			srcp[1] = 0x89ABCDEF;
		}
		
		if(!restart){	/* ---- restart is false ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc (ALT_DMA_CHANNEL_t channel);	*/
			util_time_record_start_point(topidx + 0);
			result_code = alt_dma_channel_alloc(channel);
			util_time_record_end_point(topidx + 0);
			printf("alt_dma_channel_alloc(%d) .... result=%d\n", (int)channel, (int)result_code);
			
			util_time_record_start_point(topidx + 1);
			__asm("CPSID i");
			result_code = sample_dmac_single_transfer(channel, &dma_program[channel], test_p->dst_address, test_p->src_address, 
				(test_p->trans_size/(test_p->width_bits>>3)), test_p->width_bits, dst_inc, src_inc, true, event);
			util_time_record_end_point(topidx + 1);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("sample_dmac_single_transfer(dst_inc=%s, src_inc=%s) .... result=%d\n", ((dst_inc == true )?"true":"false"), ((src_inc == true )?"true":"false"), (int)result_code);
			printf("+-- CH=%d DST=0x%08X SRC=0x%08X SIZE=0x%08X WIDTH=%d\n", (int)channel, (int)test_p->dst_address, (int)test_p->src_address, (int)test_p->trans_size, (int)test_p->width_bits);
		} else {	/* ---- restart is true ---- */
			/* Usage:	ALT_STATUS_CODE  alt_dma_channel_exec (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *pgm);	*/
			util_time_record_start_point(topidx + 2);
			__asm("CPSID i");
			result_code = alt_dma_channel_exec(channel, &dma_program[channel]);
			util_time_record_end_point(topidx + 2);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: ");
			} else {
				util_time_record_start_point(topidx + 3);
			}
			__asm("CPSIE i");
			printf("alt_dma_channel_exec() .... result=%d\n", (int)result_code);
		}
		break;
	default:	/* case DMATYPEe_NON: */	// ------------------------------- 
		/* NOP */
		break;
	}
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_dmac_callback_xxxx(uint32_t icciar, void *context)
 * @brief  DMA interrupt callback functions.
 *
 * @details  working opportunity an interrupt from DMAC, indicate
 *           an interrupt, and the interrupt pending bit clear.
 * @param[in] icciar  The callback to use when the given interrupt is issued.
 * @param[in] context  The user provided context.
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
void sample_dmac_callback_DMA_IRQ0(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(3);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_0) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ0) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[0]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ0 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[0]);
	return;
}
void sample_dmac_callback_DMA_IRQ1(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(7);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_1) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ1) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[1]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ1 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[1]);
	return;
}
void sample_dmac_callback_DMA_IRQ2(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(11);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_2);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_2) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ2);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ2) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[2]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ2 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[2]);
	return;
}
void sample_dmac_callback_DMA_IRQ3(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(15);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_3);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_3) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ3);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ3) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[3]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ3 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[3]);
	return;
}
void sample_dmac_callback_DMA_IRQ4(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(19);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_4);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_4) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ4);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ4) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[4]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ4 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[4]);
	return;
}
void sample_dmac_callback_DMA_IRQ5(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(23);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_5);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_5) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ5);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ5) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[5]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ5 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[5]);
	return;
}
void sample_dmac_callback_DMA_IRQ6(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(27);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_6);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_6) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ6);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ6) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[6]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ6 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[6]);
	return;
}
void sample_dmac_callback_DMA_IRQ7(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	util_time_record_end_point(31);	/* = channel * 4 + 3 */
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_7);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_7) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ7);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ7) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[7]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ7 is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[7]);
	return;
}
void sample_dmac_callback_DMA_IRQ_ABORT(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	// DMAC Pending Clear
	/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
	result_code = alt_dma_int_clear(ALT_DMA_EVENT_ABORT);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_dma_int_clear(ALT_DMA_EVENT_ABORT) result=%d\n", (int)result_code);
	}
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ_ABORT);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_IRQ_ABORT) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[8]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_IRQ_ABORT is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[8]);

	sample_dmac_print_all_status();

	return;
}
void sample_dmac_callback_DMA_ECC_CORRECTED_IRQ(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_ECC_CORRECTED_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_ECC_CORRECTED_IRQ) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[9]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_ECC_CORRECTED_IRQ is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[9]);
	return;
}
void sample_dmac_callback_DMA_ECC_UNCORRECTED_IRQ(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;
	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_ECC_UNCORRECTED_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_pending_clear(ALT_INT_INTERRUPT_DMA_ECC_UNCORRECTED_IRQ) result=%d\n", (int)result_code);
	}
	// Indicates the interrupt occurred.
	SAMPLE_CallBackCount[10]++;
	printf("[INTERRUPT] ALT_INT_INTERRUPT_DMA_ECC_UNCORRECTED_IRQ is occurred!! icciar=0x%08X counter=%d\n", (int)icciar, SAMPLE_CallBackCount[10]);
	return;
}

/********************************************************************************//*!
 * @fn  void sample_dmac_setting_gic(void)
 * @brief  Setting of GIC (DMA interrupt).
 *
 * @details    In order to receive a DMA interrupt setting of GIC.
 * @param[in,out] void  none
 * @retval void  none
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       nothing.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
void sample_dmac_setting_gic(void)
{
	ALT_STATUS_CODE result_code;

    //	ALT_INT_INTERRUPT_DMA_IRQ0                = 136, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ1                = 137, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ2                = 138, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ3                = 139, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ4                = 140, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ5                = 141, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ6                = 142, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ7                = 143, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_IRQ_ABORT           = 144, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_ECC_CORRECTED_IRQ   = 145, /*!< # */
    //	ALT_INT_INTERRUPT_DMA_ECC_UNCORRECTED_IRQ = 146,
    //	/*!<
    //	 * Interrupts sourced from the DMA Controller.
    //	 *  * All interrupts in this group are level triggered.
    //	 */

	int i;

	for(i=ALT_INT_INTERRUPT_DMA_IRQ0; i<=ALT_INT_INTERRUPT_DMA_ECC_UNCORRECTED_IRQ; i++)
	{
		/* Usage: ALT_STATUS_CODE alt_int_isr_register(ALT_INT_INTERRUPT_t int_id, alt_int_callback_t callback, void* context);	*/
		result_code = alt_int_isr_register((ALT_INT_INTERRUPT_t)i, SAMPLE_CallBacks[(i - ALT_INT_INTERRUPT_DMA_IRQ0)], NULL);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: alt_int_isr_register(%d) result=%d\n", i, (int)result_code);
		}

		/* Usage: ALT_STATUS_CODE alt_int_dist_target_set(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t target);	*/
		/* ##'target=1' the specified notification interrupts to only core0.  */
		result_code = alt_int_dist_target_set((ALT_INT_INTERRUPT_t)i, 1);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: alt_int_dist_target_set(%d, 1) result=%d\n", i, (int)result_code);
		}

		/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_set(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t trigger_type);	*/
		result_code = alt_int_dist_trigger_set((ALT_INT_INTERRUPT_t)i, ALT_INT_TRIGGER_LEVEL);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: alt_int_dist_trigger_set(%d, ALT_INT_TRIGGER_LEVEL) result=%d\n", i, (int)result_code);
		}

		/* Usage: ALT_STATUS_CODE alt_int_dist_priority_set(ALT_INT_INTERRUPT_t int_id, uint32_t priority);	*/
		result_code = alt_int_dist_priority_set((ALT_INT_INTERRUPT_t)i, 0);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: alt_int_dist_priority_set(%d, 0) result=%d\n", i, (int)result_code);
		}

		// To reset the counter interrupt detection.
		SAMPLE_CallBackCount[(i - ALT_INT_INTERRUPT_DMA_IRQ0)] = 0;

		if(i <= ALT_INT_INTERRUPT_DMA_IRQ_ABORT)
		{
			// To specify the notification methods (send events or interrupt notification) for each DMA resources of the DMA controller.
			/* Usage:	ALT_STATUS_CODE  alt_dma_event_int_select (ALT_DMA_EVENT_t evt_num, ALT_DMA_EVENT_SELECT_t opt);	*/
			result_code = alt_dma_event_int_select((ALT_DMA_EVENT_t)(ALT_DMA_EVENT_0 + i - ALT_INT_INTERRUPT_DMA_IRQ0), ALT_DMA_EVENT_SELECT_SIG_IRQ);
			if(result_code != ALT_E_SUCCESS){
				printf("ERROR!!: alt_dma_event_int_select(%d, ALT_DMA_EVENT_SELECT_SIG_IRQ) result=%d\n", (int)(ALT_DMA_EVENT_0 + i - ALT_INT_INTERRUPT_DMA_IRQ0), (int)result_code);
			}
		}

		/* No need to call the API below. Because all interrupts are secure by default. */
		/* Usage: ALT_STATUS_CODE  alt_int_dist_secure_enable (ALT_INT_INTERRUPT_t int_id);	*/

		/* Usage: ALT_STATUS_CODE  alt_int_dist_enable (ALT_INT_INTERRUPT_t int_id);	*/
		result_code = alt_int_dist_enable((ALT_INT_INTERRUPT_t)i);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: alt_int_dist_enable(%d) result=%d\n", i, (int)result_code);
		}

	}

	return;
}

/********************************************************************************//*!
 * @fn  ALT_STATUS_CODE sample_dmac_single_transfer(ALT_DMA_CHANNEL_t channel,
                                           ALT_DMA_PROGRAM_t * program,
                                           void * dst_address,
                                           const void * src_address,
                                           size_t count,
                                           uint32_t width_bits,
                                           bool dst_inc;
                                           bool src_inc;
                                           bool send_evt,
                                           ALT_DMA_EVENT_t evt)
 * @brief  DMA Transfer on Single Burst.
 *
 * @details  Uses the DMA engine to asynchronously copy the specified memory from the
 *           given source address to the given destination address.
 * @param[in] channel  The DMA channel thread to use for the transfer.
 * @param[in,out] program  An allocated DMA program buffer to use for the life of the transfer.
 * @param[in] dst_address  The destination address to copy to.
 * @param[in] src_address  The source address to copy from.
 * @param[in] count  The number of transfers to make. (max. 65536 + 255)
 * @param[in] width_bits  The width of the register to transfer to in bits. Valid values are 8, 16, 32, and 64.
 * @param[in] dst_inc  If set to true, Increment the destination address when it writes the destination data.
 * @param[in] src_inc  If set to true, Increment the source address when it reads the source data.
 * @param[in] send_evt  If set to true, the DMA engine will be instructed to send an event upon completion or fault.
 * @param[in] evt  If send_evt is true, the event specified will be sent. Otherwise the parameter is ignored.
 * @retval ALT_E_SUCCESS   The operation was successful.
 * @retval ALT_E_ERROR     The operation failed.
 * @retval ALT_E_BAD_ARG   The given channel or event identifier (if
 *                              used) is invalid, or the memory regions
 *                              specified are overlapping.
 *
 * @attention  nothing.
 * @pre        nothing.
 * @post       nothing.
 * @note       Created based on HWLib API 'alt_dma_register_to_memory'.
 *
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/03/04  ---  #13246  SingleBurstwDMA]ǉ.
 **//*******************************************************************************/
ALT_STATUS_CODE sample_dmac_single_transfer(ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t* program,
		void* dst_address, const void* src_address, size_t count, uint32_t width_bits,
		bool dst_inc, bool src_inc, bool send_evt, ALT_DMA_EVENT_t evt)
{
    ALT_STATUS_CODE status = ALT_E_SUCCESS;

    // If the count is zero, and no event is requested, just return success.
    if ((count == 0) && (send_evt == false))
    {
        return status;
    }

    if (status == ALT_E_SUCCESS)
    {
        status = alt_dma_program_init(program);
    }
    
    if (count != 0)
    {
        // Verify valid width_bits and construct the CCR SS and DS parameters.
        uint32_t ccr_ss_ds_mask = 0;

        if (status == ALT_E_SUCCESS)
        {
            switch (width_bits)
            {
            case 8:
                // Program in the following parameters:
                //  - SS8 (Source      burst size of 8 bits)
                //  - DS8 (Destination burst size of 8 bits)
                ccr_ss_ds_mask = ALT_DMA_CCR_OPT_SS8 | ALT_DMA_CCR_OPT_DS8;
                break;
            case 16:
                // Program in the following parameters:
                //  - SS16 (Source      burst size of 16 bits)
                //  - DS16 (Destination burst size of 16 bits)
                ccr_ss_ds_mask = ALT_DMA_CCR_OPT_SS16 | ALT_DMA_CCR_OPT_DS16;
                break;
            case 32:
                // Program in the following parameters:
                //  - SS32 (Source      burst size of 32 bits)
                //  - DS32 (Destination burst size of 32 bits)
                ccr_ss_ds_mask = ALT_DMA_CCR_OPT_SS32 | ALT_DMA_CCR_OPT_DS32;
                break;
            case 64:
                // Program in the following parameters:
                //  - SS64 (Source      burst size of 64 bits)
                //  - DS64 (Destination burst size of 64 bits)
                ccr_ss_ds_mask = ALT_DMA_CCR_OPT_SS64 | ALT_DMA_CCR_OPT_DS64;
                break;
            default:
                printf("ERROR!!: DMA: Invalid register width.\n");
                status = ALT_E_BAD_ARG;
                break;
            }
        }
        
        if(dst_inc)	ccr_ss_ds_mask |= ALT_DMA_CCR_OPT_DAI;
        if(src_inc)	ccr_ss_ds_mask |= ALT_DMA_CCR_OPT_SAI;

        if(count > (65536 + 255))
        {
            printf("ERROR!!: DMA: Count value is too large.\n");
            status = ALT_E_BAD_ARG;
        }

        // Verify that the dst_address and src_address are aligned to the register width
        if (status == ALT_E_SUCCESS)
        {
            if      (((uintptr_t)dst_address & ((width_bits >> 3) - 1)) != 0)
            {
                printf("ERROR!!: DMA: Destination Address is not aligned.\n");
                status = ALT_E_BAD_ARG;
            }
            else if (((uintptr_t)src_address & ((width_bits >> 3) - 1)) != 0)
            {
                printf("ERROR!!: DMA: Source Address is not aligned.\n");
                status = ALT_E_BAD_ARG;
            }
        }

        if (status == ALT_E_SUCCESS)
        {
            status = alt_dma_program_DMAMOV(program, ALT_DMA_PROGRAM_REG_SAR, (uint32_t)src_address);
        }
        if (status == ALT_E_SUCCESS)
        {
            status = alt_dma_program_DMAMOV(program, ALT_DMA_PROGRAM_REG_DAR, (uint32_t)dst_address);
        }
        if (status == ALT_E_SUCCESS)
        {
	        //
	        // The algorithm uses the strategy described in PL330 B.2.3.
	        // Not sure if registers will accept burst transfers so read the register in its own transfer.
	        //

	        // Program in the following parameters:
	        //  - SAX  (Source      address incremental .. Specified by [ccr_ss_ds_mask])
	        //  - SSx  (Source      burst size of [ccr_ss_ds_mask])
	        //  - DAX  (Destination address incremental .. Specified by [ccr_ss_ds_mask])
	        //  - DSx  (Destination burst size of [ccr_ss_ds_mask])
	        //  - SB1  (Source      burst length of 1 (Single Transfer))
	        //  - DB1  (Destination burst length of 1 (Single Transfer))
	        //  - All other options default.

            status = alt_dma_program_DMAMOV(program, ALT_DMA_PROGRAM_REG_CCR,
                                            (   ccr_ss_ds_mask
                                              | ALT_DMA_CCR_OPT_SB1
                                              | ALT_DMA_CCR_OPT_SP_DEFAULT
                                              | ALT_DMA_CCR_OPT_SC_DEFAULT
                                              | ALT_DMA_CCR_OPT_DB1
                                              | ALT_DMA_CCR_OPT_DP_DEFAULT
                                              | ALT_DMA_CCR_OPT_DC_DEFAULT
                                              | ALT_DMA_CCR_OPT_ES_DEFAULT
                                            )
                );	/* DMACݒiRg[WX^j */
        }

        // Calculates the number of loops from the number of transfers.
        uint32_t countx256 = count >> 8;
        uint32_t countleft = count & (256 - 1);

		if(countx256 != 0)
		{
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALP(program, countx256);							/* loop1 Jn */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALP(program, 256);								/* loop2 Jn */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALD(program, ALT_DMA_PROGRAM_INST_MOD_NONE);		/* [h */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMAST(program, ALT_DMA_PROGRAM_INST_MOD_NONE);		/* XgA */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALPEND(program, ALT_DMA_PROGRAM_INST_MOD_NONE);	/* loop2 I[ */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALPEND(program, ALT_DMA_PROGRAM_INST_MOD_NONE);	/* loop1 I[ */
			}
		}
		
		if(countleft != 0)
		{
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALP(program, countleft);							/* loop1 Jn */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALD(program, ALT_DMA_PROGRAM_INST_MOD_NONE);		/* [h */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMAST(program, ALT_DMA_PROGRAM_INST_MOD_NONE);		/* XgA */
			}
			if (status == ALT_E_SUCCESS)
			{
			    status = alt_dma_program_DMALPEND(program, ALT_DMA_PROGRAM_INST_MOD_NONE);	/* loop1 I[ */
			}
		}
	}

    // Send event if requested.
    if (send_evt)
    {
        if (status == ALT_E_SUCCESS)
        {
            dprintf("DMA: Adding event ...\n");
            status = alt_dma_program_DMASEV(program, evt);	/* ]Cxgs */
        }
    }

    // Now that everything is done, end the program.
    if (status == ALT_E_SUCCESS)
    {
        status = alt_dma_program_DMAEND(program);		/* DMAvOI */
    }

    // If there was a problem assembling the program, clean up the buffer and exit.
    if (status != ALT_E_SUCCESS)
    {
        // Do not report the status for the clear operation. A failure should be
        // reported regardless of if the clear is successful.
        alt_dma_program_clear(program);
        return status;
    }

    // Execute the program on the given channel.
    return alt_dma_channel_exec(channel, program);
}

/********************************************************************************//*!
 * @fn  void sample_test_init(void) 
 * @brief  Initializing the sample program. 
 * 
 * @details  Call the initialization function for the DMA Controller API. 
 *           Other, do the following.
 *           -# Configure the Helio Board GPIO settings. 
 *           -# Setting for the L2 Cache Address Filter. 
 *           -# Setting for the Address Space Remap Register. 
 *           -# Execute the FPGA interface enable API. 
 *           -# Execute the DMA CH allocation API for all CHs. 
 *           -# GIC Setting for DMA Interrupts. 
 *           -# Writing the Test Data to DMA Src/Dst Memory. 
 *           -# Execute the send DMA Event API for all CHs. (TEST for Interrupt setting).
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
void sample_test_init(void)
{
	ALT_STATUS_CODE result_code;
	int i;
	
	/* Configure the Helio Board GPIO settings. */
	sample_gpio_config_helioboard();
	
	/* Setting for the L2 Cache Address Filter. */
	sample_dmac_setting_l2addrfilter(0, 0xC0000000);
	/* Setting for the Address Space Remap Register. */
	sample_dmac_setting_remap();
	
	printf("==== HWLib test : Execute the FPGA interface enable/disable API ====\n");
	printf("-- execute enable API -- (Execute alt_fpga_interface_enable)\n");
	sample_sysmgr_enable_all_fpga_interface();
	printf("-- print current settings -- (Execute alt_fpga_interface_is_enabled)\n");
	sample_sysmgr_print_all_fpga_interface();
	printf("\n");

	/* Usage:	ALT_STATUS_CODE  alt_dma_init (const ALT_DMA_CFG_t *dma_cfg);	*/
    // Set the parameters for alt_dma_init().
    for(i=0; i<4; i++){
        dma_config.periph_mux[i] = ALT_DMA_PERIPH_MUX_DEFAULT;	// Handle FPGA / CAN muxing (default is FPGA)
    }
    dma_config.manager_sec = ALT_DMA_SECURITY_DEFAULT;			// Handle Manager Secure / NonSecure (default is Secure)
    for(i=0; i<ALT_SYSMGR_DMA_CTL_IRQNONSECURE_WIDTH; i++){
        dma_config.irq_sec[i] = ALT_DMA_SECURITY_DEFAULT;		// Handle IRQ Secure / NonSecure (default is Secure)
    }
    for(i=0; i<32; i++){
        dma_config.periph_sec[i] = ALT_DMA_SECURITY_DEFAULT;	// peripheral Secure / NonSecure (default is Secure)
    }
	printf("-- HWLib test : alt_dma_init() --\n");
	result_code = alt_dma_init(&dma_config);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_dma_init() .... result=%d\n", (int)result_code);
	
    /* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc (ALT_DMA_CHANNEL_t channel);	*/
	for(i=ALT_DMA_CHANNEL_0; i<=ALT_DMA_CHANNEL_7; i++){
        result_code = alt_dma_channel_alloc((ALT_DMA_CHANNEL_t)i);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_dma_channel_alloc(%d) .... result=%d\n", i, (int)result_code);
    }

	/* GIC Setting for DMA Interrupts */
	sample_dmac_setting_gic();

	// Setting TEST Data!!! //
	sample_dmac_setting_testdata((uint32_t*)0x04000000, (uint32_t*)0x06000000);

	/* Usage:	ALT_STATUS_CODE  alt_dma_send_event (ALT_DMA_EVENT_t evt_num);	*/
	for(i=ALT_DMA_EVENT_0; i<=ALT_DMA_EVENT_7; i++){
		util_time_record_start_point((i * 4 + 3));	/* = channel * 4 + 3 */
        result_code = alt_dma_send_event((ALT_DMA_EVENT_t)i);
		if(result_code != ALT_E_SUCCESS){
			printf("ERROR!!: ");
		}
		printf("alt_dma_send_event(%d) .... result=%d\n", i, (int)result_code);
    }
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_uninit(void) 
 * @brief  Uninitializing the sample program. 
 *           Other, do the following.
 *           -# Execute the FPGA interface disable API. 
 * 
 * @details  Call the uninitialization function for the DMA Controller API.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
void sample_test_uninit(void)
{
	ALT_STATUS_CODE result_code;
	
	printf("==== HWLib test : Execute the FPGA interface disable API ====\n");
	printf("-- execute disable API -- (Execute alt_fpga_interface_disable)\n");
	sample_sysmgr_disable_all_fpga_interface();
	printf("-- print current settings -- (Execute alt_fpga_interface_is_enabled)\n");
	sample_sysmgr_print_all_fpga_interface();
	printf("\n");

	/* Usage:	ALT_STATUS_CODE  alt_dma_uninit (void);	*/
	printf("-- HWLib test : alt_dma_uninit() --\n");
	result_code = alt_dma_uninit();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: ");
	}
	printf("alt_dma_uninit() .... result=%d\n", (int)result_code);

	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_print_usage(void) 
 * @brief  Display usage of the sample program. 
 * 
 * @details  Following information displays on the console.
 *            - Overview of HPS for push button functions (or operations)
 *            - Overview of HPS for DIP switch functions (or operations)
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/08/18  ---  #xxxxx  VK쐬. 
 **//*******************************************************************************/
void sample_test_print_usage(void)
{
	printf("+--- Usage ( Function of HPS-PUSHSW and HPS-DIPSW )  -----------+\n");
	printf("+ PUSHSW HPS 0(SW8) .... Exit Test loop!!!\n");
	printf("+ PUSHSW HPS 1(SW10) ... Run the API:alt_dma_program_update_reg() as testing.\n");
	printf("+ PUSHSW HPS 2(SW7) .... Display result of DMA CH Status and transfer time.\n");
	printf("+ PUSHSW HPS 3(SW9) .... Start DMA transfer.\n");
	printf("+ DIPSW HPS(SW14) 1 .... Select the first start(SW=ON) or restart(OFF).\n");
	printf("   If you select restart, skipping the DMAC micro code generation process.");
	printf("+ DIPSW HPS(SW14) 2-4 .. Select DMA channel. (000=CH0,001=CH1,..,111=CH7)\n");
	printf("+---------------------------------------------------------------+\n\n");
	
#if 0
	[vHPSpPUSHSW̑oꍇAꂼȉ̏s܂B
		- HPSpPUSHSW0 ... [v𔲂ăvOI܂B(uninits)
		- HPSpPUSHSW1 ... APĨeXgualt_dma_program_update_regv(#4)(#5)
		- HPSpPUSHSW2 ... CHԂDMA]̌vʂ\B(#4)
		- HPSpPUSHSW3 ... DMA]s (#4)(#6)

		#4 .. HPSpDIPSW[2:4]DMA CH0`7I܂B
		#5 .. API𗘗pē]AhX(SAR)Ɠ]AhX(DAR)܂B
		@@@(āAPUSHSW3ƃAhXDMA]̊mFs܂B)
		#6 .. HPSpDIPSW[1]DMA] N(OFF) or ċN(ON) w肵܂B
		@@@(DMA CHɁAŏ͕K N(OFF) ɐݒ肵Ďsĉ)
#endif
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_main(void) 
 * @brief  Main routine of the sample program. 
 * 
 * @details  Infinite loop structure works. Make the detection of various types 
 *           of switches in the loop, it performs in various testing processes.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/26  ---  #13246  VK쐬. 
 **//*******************************************************************************/
void sample_test_main(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t ui32portc = 0;
	uint32_t ui32portcbk = 0xFFFFFFFF;
	uint32_t ui32portc_onchg = 0;
	uint32_t ui32dipsw = 0;
	ALT_DMA_CHANNEL_t channel;
	ALT_DMA_EVENT_t event;
	int dma_testnum;
	
	/* ################################################ */
	/* ## Implement the test setting process here!!! ## */
	/* ################################################ */
	// DMA Programming for all DMA CH.
	printf("\nDo DMA programming and transfer execution for all DMA CH.\n");
	for(dma_testnum=ALT_DMA_EVENT_0; dma_testnum<=ALT_DMA_EVENT_7; dma_testnum++){
		// DMA Start!!
		sample_dmac_transfer_start(ALT_DMA_CHANNEL_0+dma_testnum, dma_testnum, &dma_setting[dma_testnum], false);
    }

	printf("\n==== Start While(1) loop process!!! (Exit PUSHSW0(SW8) becomes ON.) ====\n");
	sample_test_print_usage();
	while(1)
	{
		// ---- Check the DIPSW and PUSHSW. ----
		ui32portc = alt_gpio_port_data_read(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL);
		ui32portc = ~ui32portc & ALT_GPIO_PORTBIT_SWALL;	// bit inversion and mask ({ON=0,OFF=1}->{ON=1,OFF=0})
		ui32portc_onchg |= (ui32portc ^ ui32portcbk) & ui32portc;

		// DIPSW HPS 1-4
		if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPSWALL)
		{
			ui32dipsw = (uint32_t)((ui32portc & ALT_GPIO_PORTBIT_DIPSWALL) >> ALT_GPIO_LSBBIT_DIPSWS);
			ui32dipsw *= 1;		// To avoid warnings.

			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS1){
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS2){
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS3){
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS4){
			}
			
			// Select test settings from DIPSW value. 
			dma_testnum = (int)(ui32dipsw & 7);
			switch(dma_testnum)
			{
			case 0:
				channel = ALT_DMA_CHANNEL_0;
				event = ALT_DMA_EVENT_0;
				break;
			case 1:
				channel = ALT_DMA_CHANNEL_1;
				event = ALT_DMA_EVENT_1;
				break;
			case 2:
				channel = ALT_DMA_CHANNEL_2;
				event = ALT_DMA_EVENT_2;
				break;
			case 3:
				channel = ALT_DMA_CHANNEL_3;
				event = ALT_DMA_EVENT_3;
				break;
			case 4:
				channel = ALT_DMA_CHANNEL_4;
				event = ALT_DMA_EVENT_4;
				break;
			case 5:
				channel = ALT_DMA_CHANNEL_5;
				event = ALT_DMA_EVENT_5;
				break;
			case 6:
				channel = ALT_DMA_CHANNEL_6;
				event = ALT_DMA_EVENT_6;
				break;
			default:	/* case 7: */
				channel = ALT_DMA_CHANNEL_7;
				event = ALT_DMA_EVENT_7;
				break;
			}
		}

		// PUSHSW HPS 0
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH0)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH0;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			sample_dmac_print_all_status();
			break;	// Exit PUSHSW0(SW8) becomes ON.
			
		}
		// PUSHSW HPS 1
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH1)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH1;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			// Update DMA Register (SAR/DAR) .. Replace the address of the "Source" and "Destination". 
			if(dma_setting[dma_testnum].type == DMATYPEe_M2M){
				/* Usage:	ALT_STATUS_CODE  alt_dma_program_update_reg (ALT_DMA_PROGRAM_t *pgm, ALT_DMA_PROGRAM_REG_t reg, uint32_t val);	*/
				result_code = alt_dma_program_update_reg(&dma_program[channel], ALT_DMA_PROGRAM_REG_SAR, (uint32_t)(dma_setting[dma_testnum].dst_address));
				if(result_code != ALT_E_SUCCESS){
					printf("ERROR!!: ");
				}
				printf("alt_dma_program_update_reg(SAR) .... result=%d\n", (int)result_code);
				
				result_code = alt_dma_program_update_reg(&dma_program[channel], ALT_DMA_PROGRAM_REG_DAR, (uint32_t)(dma_setting[dma_testnum].src_address));
				if(result_code != ALT_E_SUCCESS){
					printf("ERROR!!: ");
				}
				printf("alt_dma_program_update_reg(DAR) .... result=%d\n", (int)result_code);
				printf("\nReplaced the SAR and DAR!!!\n");
				printf("Progress value[%%] will not display correctly, until you start the next DMA transfer.\n\n");
			} else {
				printf("\nThe target of this operation is only set to DMATYPEe_M2M channels.\n\n");
			}
		}
		// PUSHSW HPS 2
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH2)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH2;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			// Print DMA CH Status 
			int topidx = channel * 4;
			
			sample_dmac_print_ch_status(channel, true);
			util_time_print_result_partial(topidx, topidx+3, enumUTIL_TIME_TARGET_by_SECOND);
			
		}
		// PUSHSW HPS 3
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH3)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH3;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			// DMA Start!!
			sample_dmac_transfer_start(channel, event, &dma_setting[dma_testnum], ((ui32dipsw & 8) == 0 ? false : true));
		}
		
		ui32portcbk = ui32portc;
	}
	printf("==== End While(1) loop process. (Exit PUSHSW0(SW8) becomes ON.) ====\n");


	return;
}

// All APIs to be tested in this sample.
/* ---- DMA Controller API ---- */
/* -- DMA API for Configuration, Control, and Status -- */
/* Usage:	ALT_STATUS_CODE  alt_dma_init (const ALT_DMA_CFG_t *dma_cfg);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_uninit (void);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc (ALT_DMA_CHANNEL_t channel);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_alloc_any (ALT_DMA_CHANNEL_t *allocated);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_free (ALT_DMA_CHANNEL_t channel);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_exec (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *pgm);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_kill (ALT_DMA_CHANNEL_t channel);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_reg_get (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_REG_t reg, uint32_t *val);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_send_event (ALT_DMA_EVENT_t evt_num);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_manager_state_get (ALT_DMA_MANAGER_STATE_t *state);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_state_get (ALT_DMA_CHANNEL_t channel, ALT_DMA_CHANNEL_STATE_t *state);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_manager_fault_status_get (ALT_DMA_MANAGER_FAULT_t *fault);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_channel_fault_status_get (ALT_DMA_CHANNEL_t channel, ALT_DMA_CHANNEL_FAULT_t *fault);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_event_int_select (ALT_DMA_EVENT_t evt_num, ALT_DMA_EVENT_SELECT_t opt);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_event_int_status_get_raw (ALT_DMA_EVENT_t evt_num);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_int_status_get (ALT_DMA_EVENT_t irq_num);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_int_clear (ALT_DMA_EVENT_t irq_num);	*/
/* -- DMA API for Standard Operations -- */
/* Usage:	ALT_STATUS_CODE  alt_dma_memory_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *dest, const void *src, size_t size, bool send_evt, ALT_DMA_EVENT_t evt);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_zero_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *buf, size_t size, bool send_evt, ALT_DMA_EVENT_t evt);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_memory_to_register (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *dst_reg, const void *src_buf, size_t count, uint32_t register_width_bits, bool send_evt, ALT_DMA_EVENT_t evt);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_register_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *dst_buf, const void *src_reg, size_t count, uint32_t register_width_bits, bool send_evt, ALT_DMA_EVENT_t evt);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_memory_to_periph (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, ALT_DMA_PERIPH_t dest, const void *src, size_t size, void *periph_info, bool send_evt, ALT_DMA_EVENT_t evt);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_periph_to_memory (ALT_DMA_CHANNEL_t channel, ALT_DMA_PROGRAM_t *program, void *dest, ALT_DMA_PERIPH_t src, size_t size, void *periph_info, bool send_evt, ALT_DMA_EVENT_t evt);	*/
/* ---- DMA Controller Programming API (Part of APIs) ---- */
/* Usage:	ALT_STATUS_CODE  alt_dma_program_progress_reg (ALT_DMA_PROGRAM_t *pgm, ALT_DMA_PROGRAM_REG_t reg, uint32_t current, uint32_t *progress);	*/
/* Usage:	ALT_STATUS_CODE  alt_dma_program_update_reg (ALT_DMA_PROGRAM_t *pgm, ALT_DMA_PROGRAM_REG_t reg, uint32_t val);	*/

/* -- Address Space Mapping Control -- */
/* Usage:	ALT_STATUS_CODE  alt_addr_space_remap (ALT_ADDR_SPACE_MPU_ATTR_t mpu_attr, ALT_ADDR_SPACE_NONMPU_ATTR_t nonmpu_attr, ALT_ADDR_SPACE_H2F_BRIDGE_ATTR_t h2f_attr, ALT_ADDR_SPACE_LWH2F_BRIDGE_ATTR_t lwh2f_attr);	*/
/* Usage:	ALT_STATUS_CODE  alt_mpu_addr_space_remap_0_to_sdram (void);	*/
/* -- L2 Cache Address Filter -- */
/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_get (uint32_t *addr_filt_start, uint32_t *addr_filt_end);	*/
/* Usage:	ALT_STATUS_CODE  alt_l2_addr_filter_cfg_set (uint32_t addr_filt_start, uint32_t addr_filt_end);	*/

////////////////////////////// Common Functions //////////////////////////////////////// 
/********************************************************************************//*!
 * @fn  static void sample_gpio_config_helioboard(void) 
 * @brief  Configure the Helio Board GPIO settings. 
 * 
 * @details  Performs the following processing. 
 *           - Gets the contents of the current port settings to gpio_config_array. 
 *           - Global timer counter value of the measurement at the start
 *           - Setting DIPSW and PUSHSW via bit index configuration func. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/02/13  ---  #13144  VK쐬. 
 **//*******************************************************************************/
static void sample_gpio_config_helioboard(void)
{
	ALT_STATUS_CODE result_code;
	ALT_GPIO_CONFIG_RECORD_t gpio_config_array[10];
	ALT_GPIO_CONFIG_RECORD_t* p_gpiocfg;

	// Setting OUT-PORTiGPIO61j via normal configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_config(	ALT_GPIO_PORT_t gpio_pid,
	 * 												uint32_t mask,
	 * 												ALT_GPIO_PIN_DIR_t dir,
	 * 												ALT_GPIO_PIN_TYPE_t type,
	 * 												ALT_GPIO_PIN_POL_t pol,
	 * 												ALT_GPIO_PIN_DEBOUNCE_t debounc,
	 * 												uint32_t data); 						*/
	/****************************************************************************************/
	result_code = alt_gpio_port_config(ALT_GPIO_PORTC, ALT_GPIO_BIT3,
			ALT_GPIO_PIN_OUTPUT, ALT_GPIO_PIN_LEVEL_TRIG_INT, ALT_GPIO_PIN_ACTIVE_HIGH, ALT_GPIO_PIN_NODEBOUNCE,
			ALT_GPIO_BIT3);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_config() result=%d\n", (int)result_code);
	}

	// Gets the contents of the current port settings to gpio_config_array.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config_get(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len);	*/
	/****************************************************************************************/
	gpio_config_array[0].signal_number = ALT_GPIO_1BIT_61;
	gpio_config_array[1].signal_number = ALT_HLGPI_6;
	gpio_config_array[2].signal_number = ALT_HLGPI_7;
	gpio_config_array[3].signal_number = ALT_HLGPI_8;
	gpio_config_array[4].signal_number = ALT_HLGPI_9;
	gpio_config_array[5].signal_number = ALT_HLGPI_10;
	gpio_config_array[6].signal_number = ALT_HLGPI_11;
	gpio_config_array[7].signal_number = ALT_HLGPI_12;
	gpio_config_array[8].signal_number = ALT_HLGPI_13;
	gpio_config_array[9].signal_number = ALT_END_OF_GPIO_SIGNALS;
	result_code = alt_gpio_group_config_get(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config_get() result=%d\n", (int)result_code);
	}

	// Setting DIPSW and PUSHSW via bit index configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len); */
	/****************************************************************************************/
	for(int i=1; i<10; i++)
	{
		p_gpiocfg = &gpio_config_array[i];
		p_gpiocfg->direction = ALT_GPIO_PIN_INPUT;
		p_gpiocfg->type = ALT_GPIO_PIN_LEVEL_TRIG_INT;
		p_gpiocfg->polarity = ALT_GPIO_PIN_ACTIVE_HIGH;
		p_gpiocfg->debounce = ALT_GPIO_PIN_NODEBOUNCE;
		p_gpiocfg->data = ALT_GPIO_PIN_DATAZERO;
	}
	result_code = alt_gpio_group_config(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config() result=%d\n", (int)result_code);
	}

	return;
}

/***********************************************************************************
 * end of file 
 ***********************************************************************************/
