/********************************************************************************//*!
 * @file  sample_gpio.c 
 * @brief  To use HWLib for the general purpose I/O example program. 
 * 
 * @details  Run the API category below.
 *           - The General Purpose Input/Output Manager API  
 *               - General-Purpose IO Configuration Functions  
 *               - General-Purpose IO Interrupt Functions  
 *               - General-Purpose IO via Bit Index  
 *               - General-Purpose IO Utility Functions 
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 * @date 2014/08/18  ---  -----  v14.0 (AA̕\ǉ). 
 * 
 * @attention 
 * Copyright (C) 2013-2014 ALTIMA Corp./ELSENA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

/***********************************************************************************
 *  includes 
 ***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hwlib.h"
#include "alt_generalpurpose_io.h"
#include "alt_interrupt.h"

/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  proto types 
 ***********************************************************************************/
void sample_gpio_iointerrupt(void);
void sample_gpio_callback(uint32_t icciar, void *context);
void sample_gpio_iopolling(void);
void sample_gpio_iconfig(void);
void sample_gpio_config(void);
void sample_gpio_utility(void);
void sample_test_print_usage(void);

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
#define ALT_GPIO_PORTBIT_ALL	((ALT_GPIO_BIT0) | \
								 (ALT_GPIO_BIT1) | \
								 (ALT_GPIO_BIT2) | \
								 (ALT_GPIO_BIT3) | \
								 (ALT_GPIO_BIT4) | \
								 (ALT_GPIO_BIT5) | \
								 (ALT_GPIO_BIT6) | \
								 (ALT_GPIO_BIT7) | \
								 (ALT_GPIO_BIT8) | \
								 (ALT_GPIO_BIT9) | \
								 (ALT_GPIO_BIT10) | \
								 (ALT_GPIO_BIT11) | \
								 (ALT_GPIO_BIT12) | \
								 (ALT_GPIO_BIT13) | \
								 (ALT_GPIO_BIT14) | \
								 (ALT_GPIO_BIT15) | \
								 (ALT_GPIO_BIT16) | \
								 (ALT_GPIO_BIT17) | \
								 (ALT_GPIO_BIT18) | \
								 (ALT_GPIO_BIT19) | \
								 (ALT_GPIO_BIT20) | \
								 (ALT_GPIO_BIT21) | \
								 (ALT_GPIO_BIT22) | \
								 (ALT_GPIO_BIT23) | \
								 (ALT_GPIO_BIT24) | \
								 (ALT_GPIO_BIT25) | \
								 (ALT_GPIO_BIT26) | \
								 (ALT_GPIO_BIT27) | \
								 (ALT_GPIO_BIT28))

#define ALT_GPIO_PORTBIT_DIPS1		(ALT_GPIO_BIT26)
#define ALT_GPIO_PORTBIT_DIPS2		(ALT_GPIO_BIT25)
#define ALT_GPIO_PORTBIT_DIPS3		(ALT_GPIO_BIT24)
#define ALT_GPIO_PORTBIT_DIPS4		(ALT_GPIO_BIT23)
#define ALT_GPIO_PORTBIT_DIPSWALL	((ALT_GPIO_BIT26)|(ALT_GPIO_BIT25)|(ALT_GPIO_BIT24)|(ALT_GPIO_BIT23))
#define ALT_GPIO_PORTBIT_PUSH1		(ALT_GPIO_BIT22)
#define ALT_GPIO_PORTBIT_PUSH2		(ALT_GPIO_BIT21)
#define ALT_GPIO_PORTBIT_PUSH3		(ALT_GPIO_BIT20)
#define ALT_GPIO_PORTBIT_PUSH4		(ALT_GPIO_BIT19)
#define ALT_GPIO_PORTBIT_PUSHSWALL	((ALT_GPIO_BIT22)|(ALT_GPIO_BIT21)|(ALT_GPIO_BIT20)|(ALT_GPIO_BIT19))
#define ALT_GPIO_PORTBIT_SWALL		((ALT_GPIO_PORTBIT_DIPSWALL)|(ALT_GPIO_PORTBIT_PUSHSWALL))

/** memo: port asign ***************/
/* Signal Name <--> HPS Port Name  */
/* USER_DIPSW_HPS0 <--> HPS_GPI13  */
/* USER_DIPSW_HPS1 <--> HPS_GPI12  */
/* USER_DIPSW_HPS2 <--> HPS_GPI11  */
/* USER_DIPSW_HPS3 <--> HPS_GPI10  */
/* USER_PB_HPS0 <--> HPS_GPI9      */
/* USER_PB_HPS1 <--> HPS_GPI8      */
/* USER_PB_HPS2 <--> HPS_GPI7      */
/* USER_PB_HPS3 <--> HPS_GPI6      */
/* USER_HPS_GPIO61 <--> HPS_GPIO61 */
/***********************************/

/***********************************************************************************
 *  variables 
 ***********************************************************************************/

/***********************************************************************************
 *  functions 
 ***********************************************************************************/

/********************************************************************************//*!
 * @fn  void sample_gpio_iointerrupt(void) 
 * @brief  Interrupt controller Setting for GPIO interrupts.
 * 
 * @details  In order to receive the GPIO interrupts, interrupt callback routine 
 *           registered, priority setting, configuration of the Distributor, and 
 *           interrupt enable setting.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gpio_iointerrupt(void)
{
	ALT_STATUS_CODE result_code;
	uint32_t ui32temp = 0;
	ALT_INT_TRIGGER_t trigger_type = ALT_INT_TRIGGER_NA;

	// Print initial value (GIC: GPIO2)
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_secure(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_enabled(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_pending(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_is_active(ALT_INT_INTERRUPT_t int_id);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_priority_get(ALT_INT_INTERRUPT_t int_id, uint32_t* priority);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_target_get(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t* target);	*/
	/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_get(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t* trigger_type);	*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_int_dist_is_secure() --\n");
	printf("arg1=ALT_INT_INTERRUPT_GPIO0 --> result=%d\n", (int)alt_int_dist_is_secure(ALT_INT_INTERRUPT_GPIO0));
	printf("arg1=ALT_INT_INTERRUPT_GPIO1 --> result=%d\n", (int)alt_int_dist_is_secure(ALT_INT_INTERRUPT_GPIO1));
	printf("arg1=ALT_INT_INTERRUPT_GPIO2 --> result=%d\n", (int)alt_int_dist_is_secure(ALT_INT_INTERRUPT_GPIO2));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_is_enabled() --\n");
	printf("arg1=ALT_INT_INTERRUPT_GPIO0 --> result=%d\n", (int)alt_int_dist_is_enabled(ALT_INT_INTERRUPT_GPIO0));
	printf("arg1=ALT_INT_INTERRUPT_GPIO1 --> result=%d\n", (int)alt_int_dist_is_enabled(ALT_INT_INTERRUPT_GPIO1));
	printf("arg1=ALT_INT_INTERRUPT_GPIO2 --> result=%d\n", (int)alt_int_dist_is_enabled(ALT_INT_INTERRUPT_GPIO2));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_is_pending() --\n");
	printf("arg1=ALT_INT_INTERRUPT_GPIO0 --> result=%d\n", (int)alt_int_dist_is_pending(ALT_INT_INTERRUPT_GPIO0));
	printf("arg1=ALT_INT_INTERRUPT_GPIO1 --> result=%d\n", (int)alt_int_dist_is_pending(ALT_INT_INTERRUPT_GPIO1));
	printf("arg1=ALT_INT_INTERRUPT_GPIO2 --> result=%d\n", (int)alt_int_dist_is_pending(ALT_INT_INTERRUPT_GPIO2));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_is_active() --\n");
	printf("arg1=ALT_INT_INTERRUPT_GPIO0 --> result=%d\n", (int)alt_int_dist_is_active(ALT_INT_INTERRUPT_GPIO0));
	printf("arg1=ALT_INT_INTERRUPT_GPIO1 --> result=%d\n", (int)alt_int_dist_is_active(ALT_INT_INTERRUPT_GPIO1));
	printf("arg1=ALT_INT_INTERRUPT_GPIO2 --> result=%d\n", (int)alt_int_dist_is_active(ALT_INT_INTERRUPT_GPIO2));
	printf("\n");
	printf("-- HWLib test : alt_int_dist_priority_get() --\n");
	printf("arg1=ALT_INT_INTERRUPT_GPIO0 --> result=%d out=0x%08X\n", (int)alt_int_dist_priority_get(ALT_INT_INTERRUPT_GPIO0,&ui32temp), (int)ui32temp);
	printf("arg1=ALT_INT_INTERRUPT_GPIO1 --> result=%d out=0x%08X\n", (int)alt_int_dist_priority_get(ALT_INT_INTERRUPT_GPIO1,&ui32temp), (int)ui32temp);
	printf("arg1=ALT_INT_INTERRUPT_GPIO2 --> result=%d out=0x%08X\n", (int)alt_int_dist_priority_get(ALT_INT_INTERRUPT_GPIO2,&ui32temp), (int)ui32temp);
	printf("\n");
	printf("-- HWLib test : alt_int_dist_target_get() --\n");
	printf("arg1=ALT_INT_INTERRUPT_GPIO0 --> result=%d out=0x%08X\n", (int)alt_int_dist_target_get(ALT_INT_INTERRUPT_GPIO0,&ui32temp), (int)ui32temp);
	printf("arg1=ALT_INT_INTERRUPT_GPIO1 --> result=%d out=0x%08X\n", (int)alt_int_dist_target_get(ALT_INT_INTERRUPT_GPIO1,&ui32temp), (int)ui32temp);
	printf("arg1=ALT_INT_INTERRUPT_GPIO2 --> result=%d out=0x%08X\n", (int)alt_int_dist_target_get(ALT_INT_INTERRUPT_GPIO2,&ui32temp), (int)ui32temp);
	printf("\n");
	printf("-- HWLib test : alt_int_dist_trigger_get() --\n");
	printf("arg1=ALT_INT_INTERRUPT_GPIO0 --> result=%d out=%d\n", (int)alt_int_dist_trigger_get(ALT_INT_INTERRUPT_GPIO0,&trigger_type), (int)trigger_type);
	printf("arg1=ALT_INT_INTERRUPT_GPIO1 --> result=%d out=%d\n", (int)alt_int_dist_trigger_get(ALT_INT_INTERRUPT_GPIO1,&trigger_type), (int)trigger_type);
	printf("arg1=ALT_INT_INTERRUPT_GPIO2 --> result=%d out=%d\n", (int)alt_int_dist_trigger_get(ALT_INT_INTERRUPT_GPIO2,&trigger_type), (int)trigger_type);
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_isr_register(ALT_INT_INTERRUPT_t int_id, alt_int_callback_t callback, void* context);	*/
	/*  typedef void(* alt_int_callback_t)(uint32_t icciar, void *context) */
	printf("-- HWLib test : alt_int_isr_register() --\n");
	result_code = alt_int_isr_register(ALT_INT_INTERRUPT_GPIO2, sample_gpio_callback, NULL);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_isr_register() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_target_set(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t target);	*/
	printf("-- HWLib test : alt_int_dist_target_set() --\n");
	result_code = alt_int_dist_target_set(ALT_INT_INTERRUPT_GPIO0, 1);		// arg2=targetCPU (Only Core#0)
	result_code |= alt_int_dist_target_set(ALT_INT_INTERRUPT_GPIO1, 1);		// arg2=targetCPU (Only Core#0)
	result_code |= alt_int_dist_target_set(ALT_INT_INTERRUPT_GPIO2, 1);		// arg2=targetCPU (Only Core#0)
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_target_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_set(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t trigger_type);	*/
	printf("-- HWLib test : alt_int_dist_trigger_set() --\n");
	result_code = alt_int_dist_trigger_set(ALT_INT_INTERRUPT_GPIO0, ALT_INT_TRIGGER_LEVEL);
	result_code |= alt_int_dist_trigger_set(ALT_INT_INTERRUPT_GPIO1, ALT_INT_TRIGGER_LEVEL);
	result_code |= alt_int_dist_trigger_set(ALT_INT_INTERRUPT_GPIO2, ALT_INT_TRIGGER_LEVEL);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_trigger_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_priority_set(ALT_INT_INTERRUPT_t int_id, uint32_t priority);	*/
	printf("-- HWLib test : alt_int_dist_priority_set() --\n");
	result_code = alt_int_dist_priority_set(ALT_INT_INTERRUPT_GPIO0, 99);
	result_code |= alt_int_dist_priority_set(ALT_INT_INTERRUPT_GPIO1, 99);
	result_code |= alt_int_dist_priority_set(ALT_INT_INTERRUPT_GPIO2, 99);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_priority_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_enable(ALT_INT_INTERRUPT_t int_id);	*/
	printf("-- HWLib test : alt_int_dist_enable() --\n");
	result_code = alt_int_dist_enable(ALT_INT_INTERRUPT_GPIO0);
	result_code |= alt_int_dist_enable(ALT_INT_INTERRUPT_GPIO1);
	result_code |= alt_int_dist_enable(ALT_INT_INTERRUPT_GPIO2);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_enable() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_secure_enable(ALT_INT_INTERRUPT_t int_id);	*/
	printf("-- HWLib test : alt_int_dist_secure_enable() --\n");
	result_code = alt_int_dist_secure_enable(ALT_INT_INTERRUPT_GPIO0);
	result_code |= alt_int_dist_secure_enable(ALT_INT_INTERRUPT_GPIO1);
	result_code |= alt_int_dist_secure_enable(ALT_INT_INTERRUPT_GPIO2);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_enable() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  void sample_gpio_callback(uint32_t icciar, void *context) 
 * @brief  GPIO interrupt callback function.
 * 
 * @details  working opportunity an interrupt from the GPIO, indicate an 
 *           interrupt, and the interrupt pending bit clear.
 * @param[in] icciar  The callback to use when the given interrupt is issued. 
 * @param[in] context  The user provided context.  
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gpio_callback(uint32_t icciar, void *context)
{
	uint32_t ui32result;
	ALT_STATUS_CODE result_code;

	/* Usage: uint32_t alt_gpio_port_int_status_get(ALT_GPIO_PORT_t gpio_pid);				*/
	ui32result = alt_gpio_port_int_status_get(ALT_GPIO_PORTC);
	printf("[INTERRUPT]alt_gpio_port_int_status_get(ALT_GPIO_PORTC) -.-> result=0x%08X\n", (int)ui32result);
	/* Usage: ALT_STATUS_CODE alt_gpio_port_int_status_clear(ALT_GPIO_PORT_t gpio_pid, uint32_t clrmask);	*/
	result_code = alt_gpio_port_int_status_clear(ALT_GPIO_PORTC, ui32result);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT]ERROR!!: alt_gpio_port_int_status_clear() result=%d\n", (int)result_code);
	}

	/* Usage: ALT_STATUS_CODE alt_int_dist_pending_clear(ALT_INT_INTERRUPT_t int_id);	*/
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_GPIO2);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT]ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
	}

	printf("\n");
}

/********************************************************************************//*!
 * @fn  void sample_gpio_iopolling(void) 
 * @brief  Main routine of the sample program. 
 * 
 * @details  Infinite loop structure works. Make the detection of various types 
 *           of switches in the loop, it performs in various testing processes.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gpio_iopolling(void)
{
	uint32_t ui32result;
	uint32_t ui32backup;

	printf("==== Start While(1) loop process!!! (Exit DIPSW4 becomes ON or OFF.) ====\n");
	sample_test_print_usage();
	ui32backup = alt_gpio_port_data_read(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL);
	printf("Initial value ALT_GPIO_PORTC = 0x%08X\n", (int)ui32backup);
	while(1)
	{
		ui32result = alt_gpio_port_data_read(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL);
		if(ui32result != ui32backup)
		{
			printf("Change detection ALT_GPIO_PORTC = 0x%08X --> 0x%08X\n", (int)ui32backup, (int)ui32result);
		}

		if(((ui32backup ^ ui32result) & ALT_GPIO_PORTBIT_DIPS4))
		{
			break;	// Exit DIPSW4 becomes ON or OFF.
		}

		ui32backup = ui32result;
	}
	printf("==== End While(1) loop process. (Exit DIPSW4 becomes ON or OFF.) ====\n");
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_gpio_iconfig(void) 
 * @brief  Internal setting of GPIO block for interrupts. 
 * 
 * @details  In order to receive the GPIO interrupts, polarity (Edge or Level, 
 *           Rise(High) or Fall(Low)..), mask, and interrupt enable setting.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gpio_iconfig(void)
{
	uint32_t ui32result;
	ALT_STATUS_CODE result_code;

	// Print initial value
	/****************************************************************************************/
	/* Usage: uint32_t alt_gpio_port_int_type_get(ALT_GPIO_PORT_t gpio_pid, uint32_t mask);	*/
	/* Usage: uint32_t alt_gpio_port_int_pol_get(ALT_GPIO_PORT_t gpio_pid, uint32_t mask);	*/
	/* Usage: uint32_t alt_gpio_port_int_enable_get(ALT_GPIO_PORT_t gpio_pid);				*/
	/* Usage: uint32_t alt_gpio_port_int_mask_get(ALT_GPIO_PORT_t gpio_pid);				*/
	/* Usage: uint32_t alt_gpio_port_int_status_get(ALT_GPIO_PORT_t gpio_pid);				*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_port_int_type_get() --\n");
	ui32result = alt_gpio_port_int_type_get(ALT_GPIO_PORTA, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTA, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_type_get(ALT_GPIO_PORTB, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTB, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_type_get(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_pol_get() --\n");
	ui32result = alt_gpio_port_int_pol_get(ALT_GPIO_PORTA, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTA, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_pol_get(ALT_GPIO_PORTB, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTB, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_pol_get(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_enable_get() --\n");
	ui32result = alt_gpio_port_int_enable_get(ALT_GPIO_PORTA);
	printf("arg1=ALT_GPIO_PORTA --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_enable_get(ALT_GPIO_PORTB);
	printf("arg1=ALT_GPIO_PORTB --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_enable_get(ALT_GPIO_PORTC);
	printf("arg1=ALT_GPIO_PORTC --> result=0x%08X\n", (int)ui32result);
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_mask_get() --\n");
	ui32result = alt_gpio_port_int_mask_get(ALT_GPIO_PORTA);
	printf("arg1=ALT_GPIO_PORTA --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_mask_get(ALT_GPIO_PORTB);
	printf("arg1=ALT_GPIO_PORTB --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_mask_get(ALT_GPIO_PORTC);
	printf("arg1=ALT_GPIO_PORTC --> result=0x%08X\n", (int)ui32result);
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_status_get() --\n");
	ui32result = alt_gpio_port_int_status_get(ALT_GPIO_PORTA);
	printf("arg1=ALT_GPIO_PORTA --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_status_get(ALT_GPIO_PORTB);
	printf("arg1=ALT_GPIO_PORTB --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_int_status_get(ALT_GPIO_PORTC);
	printf("arg1=ALT_GPIO_PORTC --> result=0x%08X\n", (int)ui32result);
	printf("\n");

	// setting gpio interrupts
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_int_type_set(ALT_GPIO_PORT_t gpio_pid, uint32_t mask, uint32_t config); */
	/* Usage: ALT_STATUS_CODE alt_gpio_port_int_pol_set(ALT_GPIO_PORT_t gpio_pid, uint32_t mask, uint32_t config); 	*/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_int_enable(ALT_GPIO_PORT_t gpio_pid, uint32_t config);					*/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_int_disable(ALT_GPIO_PORT_t gpio_pid, uint32_t config);					*/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_int_mask_set(ALT_GPIO_PORT_t gpio_pid, uint32_t mask, uint32_t val);	*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_port_int_enable() --\n");
	result_code = alt_gpio_port_int_enable(ALT_GPIO_PORTA, 0);
	result_code |= alt_gpio_port_int_enable(ALT_GPIO_PORTB, 0);
	result_code |= alt_gpio_port_int_enable(ALT_GPIO_PORTC, 0);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_int_enable() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!! ALT_GPIO_PORTA=0x%08X / PORTB=0x%08X / PORTC=0x%08X\n",
				(int)alt_gpio_port_int_enable_get(ALT_GPIO_PORTA),
				(int)alt_gpio_port_int_enable_get(ALT_GPIO_PORTB),
				(int)alt_gpio_port_int_enable_get(ALT_GPIO_PORTC));
	}
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_type_set() --\n");
	result_code = alt_gpio_port_int_type_set(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL, ALT_GPIO_PORTBIT_SWALL);	// All Edge Sense
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_int_type_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_pol_set() --\n");
	result_code = alt_gpio_port_int_pol_set(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL,
			(ALT_GPIO_PORTBIT_PUSH1|ALT_GPIO_PORTBIT_PUSH2));	// PUSHSW1,2..RisingEdge / PUSHSW2,3..FallingEdge
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_int_pol_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_disable() --\n");
	result_code = alt_gpio_port_int_disable(ALT_GPIO_PORTA, 0);
	result_code |= alt_gpio_port_int_disable(ALT_GPIO_PORTB, 0);
	result_code |= alt_gpio_port_int_disable(ALT_GPIO_PORTC, 0);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_int_disable() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!! ALT_GPIO_PORTA=0x%08X / PORTB=0x%08X / PORTC=0x%08X\n",
				(int)alt_gpio_port_int_enable_get(ALT_GPIO_PORTA),
				(int)alt_gpio_port_int_enable_get(ALT_GPIO_PORTB),
				(int)alt_gpio_port_int_enable_get(ALT_GPIO_PORTC));
	}
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_mask_set() --\n");
	result_code = alt_gpio_port_int_mask_set(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL, ALT_GPIO_PORTBIT_DIPSWALL);	// DIPSW..Interrupt Mask
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_int_mask_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_int_enable() --\n");
	result_code = alt_gpio_port_int_enable(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_int_enable() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!! ALT_GPIO_PORTC=0x%08X\n", (int)alt_gpio_port_int_enable_get(ALT_GPIO_PORTC));
	}
	printf("\n");



	return;
}

/********************************************************************************//*!
 * @fn  void sample_gpio_config(void) 
 * @brief  GPIO configuration for the sample programs run.
 * 
 * @details  Performs the following processing. 
 *           - Setting DIPSW via normal configuration func. 
 *           - Setting OUT-PORTiGPIO61j via normal configuration func. 
 *           - Setting PUSHSW via bit index configuration func. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gpio_config(void)
{
	uint32_t ui32result;
	ALT_STATUS_CODE result_code;
	ALT_GPIO_CONFIG_RECORD_t gpio_config_array[10];
	ALT_GPIO_CONFIG_RECORD_t* p_gpiocfg;

	// Print initial value
	/****************************************************************************************/
	/* Usage: uint32_t alt_gpio_port_datadir_get(ALT_GPIO_PORT_t gpio_pid, uint32_t mask);	*/
	/* Usage: uint32_t alt_gpio_port_debounce_get(ALT_GPIO_PORT_t gpio_pid, uint32_t mask);	*/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_sync_get(ALT_GPIO_PORT_t gpio_pid);				*/
	/* Usage: uint32_t alt_gpio_port_data_read(ALT_GPIO_PORT_t gpio_pid, uint32_t mask);	*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_port_datadir_get() --\n");
	ui32result = alt_gpio_port_datadir_get(ALT_GPIO_PORTA, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTA, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_datadir_get(ALT_GPIO_PORTB, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTB, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_datadir_get(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_datadir_get() --\n");
	ui32result = alt_gpio_port_debounce_get(ALT_GPIO_PORTA, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTA, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_debounce_get(ALT_GPIO_PORTB, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTB, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_debounce_get(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_sync_get() --\n");
	printf("arg1=ALT_GPIO_PORTA --> result=%d\n", (int)alt_gpio_port_sync_get(ALT_GPIO_PORTA));
	printf("arg1=ALT_GPIO_PORTB --> result=%d\n", (int)alt_gpio_port_sync_get(ALT_GPIO_PORTB));
	printf("arg1=ALT_GPIO_PORTC --> result=%d\n", (int)alt_gpio_port_sync_get(ALT_GPIO_PORTC));
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_data_read() --\n");
	ui32result = alt_gpio_port_data_read(ALT_GPIO_PORTA, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTA, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_data_read(ALT_GPIO_PORTB, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTB, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	ui32result = alt_gpio_port_data_read(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_ALL);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALLBIT --> result=0x%08X\n", (int)ui32result);
	printf("\n");

	// setting DIPSW via normal configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_datadir_set(ALT_GPIO_PORT_t gpio_pid, uint32_t mask, uint32_t config); */
	/* Usage: ALT_STATUS_CODE alt_gpio_port_debounce_set(ALT_GPIO_PORT_t gpio_pid, uint32_t mask, uint32_t config); */
	/* Usage: ALT_STATUS_CODE alt_gpio_port_sync_set(ALT_GPIO_PORT_t gpio_pid, uint32_t config); */
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_port_datadir_set() --\n");
	result_code = alt_gpio_port_datadir_set(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_DIPSWALL, 0);	// all input port
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_datadir_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_debounce_set() --\n");
	result_code = alt_gpio_port_debounce_set(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_DIPSWALL, ALT_GPIO_PORTBIT_DIPSWALL);	// all debounce enable
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_debounce_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");
	printf("-- HWLib test : alt_gpio_port_sync_set() --\n");
	result_code = alt_gpio_port_sync_set(ALT_GPIO_PORTC, 0);	// Do not synchronize to internal clock signal.
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_sync_set() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	// setting OUT-PORTiGPIO61j via normal configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_config(	ALT_GPIO_PORT_t gpio_pid,
	 * 												uint32_t mask,
	 * 												ALT_GPIO_PIN_DIR_t dir,
	 * 												ALT_GPIO_PIN_TYPE_t type,
	 * 												ALT_GPIO_PIN_POL_t pol,
	 * 												ALT_GPIO_PIN_DEBOUNCE_t debounc,
	 * 												uint32_t data); 						*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_port_config() --\n");
	result_code = alt_gpio_port_config(ALT_GPIO_PORTC, ALT_GPIO_BIT3,
			ALT_GPIO_PIN_OUTPUT, ALT_GPIO_PIN_LEVEL_TRIG_INT, ALT_GPIO_PIN_ACTIVE_HIGH, ALT_GPIO_PIN_NODEBOUNCE,
			ALT_GPIO_BIT3);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_config() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_bitconfig_get(ALT_GPIO_1BIT_t signal_num, ALT_GPIO_CONFIG_RECORD_t* config); */
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_bitconfig_get() --\n");
	result_code = alt_gpio_bitconfig_get(ALT_GPIO_1BIT_61, &gpio_config_array[0]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_6, &gpio_config_array[1]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_7, &gpio_config_array[2]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_8, &gpio_config_array[3]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_9, &gpio_config_array[4]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_10, &gpio_config_array[5]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_11, &gpio_config_array[6]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_12, &gpio_config_array[7]);
	result_code |= alt_gpio_bitconfig_get(ALT_HLGPI_13, &gpio_config_array[8]);
	gpio_config_array[9].signal_number = ALT_END_OF_GPIO_SIGNALS;
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_bitconfig_get() result=%d\n", (int)result_code);
	} else
	{
		for(int i=0; i<10; i++)
		{
			p_gpiocfg = &gpio_config_array[i];
			printf("gpio_config_array[%d] = {sig#=%d, dire=%d, type=%d, pola=%d, debo=%d, data=%d}\n",
					i, (int)p_gpiocfg->signal_number, (int)p_gpiocfg->direction, (int)p_gpiocfg->type,
					(int)p_gpiocfg->polarity, (int)p_gpiocfg->debounce, (int)p_gpiocfg->data);
		}
	}
	printf("\n");

	/* USER_PB_HPS0 <--> HPS_GPI9      */
	/* USER_PB_HPS1 <--> HPS_GPI8      */
	/* USER_PB_HPS2 <--> HPS_GPI7      */
	/* USER_PB_HPS3 <--> HPS_GPI6      */
	// setting PUSHSW via bit index configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_bit_config(	ALT_GPIO_1BIT_t signal_num,
	 * 												ALT_GPIO_PIN_DIR_t dir,
	 * 												ALT_GPIO_PIN_TYPE_t type,
	 * 												ALT_GPIO_PIN_POL_t pol,
	 * 												ALT_GPIO_PIN_DEBOUNCE_t debounce,
	 * 												ALT_GPIO_PIN_DATA_t data);				*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_bit_config() --\n");
	result_code = alt_gpio_bit_config(ALT_HLGPI_9, ALT_GPIO_PIN_INPUT, ALT_GPIO_PIN_LEVEL_TRIG_INT,
			ALT_GPIO_PIN_ACTIVE_HIGH, ALT_GPIO_PIN_NODEBOUNCE, ALT_GPIO_PIN_DATAZERO);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_bit_config() result=%d\n", (int)result_code);
	}
	else
	{
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len); */
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_group_config() --\n");
	for(int i=1; i<10; i++)
	{
		p_gpiocfg = &gpio_config_array[i];
		p_gpiocfg->direction = ALT_GPIO_PIN_INPUT;
		p_gpiocfg->type = ALT_GPIO_PIN_LEVEL_TRIG_INT;
		p_gpiocfg->polarity = ALT_GPIO_PIN_ACTIVE_HIGH;
		p_gpiocfg->debounce = ALT_GPIO_PIN_NODEBOUNCE;
		p_gpiocfg->data = ALT_GPIO_PIN_DATAZERO;
	}
	result_code = alt_gpio_group_config(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config() result=%d\n", (int)result_code);
	}
	printf("\n");

	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config_get(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len);	*/
	/****************************************************************************************/
	printf("-- HWLib test : alt_gpio_group_config_get() --\n");
	result_code = alt_gpio_group_config_get(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config_get() result=%d\n", (int)result_code);
	} else
	{
		for(int i=0; i<10; i++)
		{
			p_gpiocfg = &gpio_config_array[i];
			printf("gpio_config_array[%d] = {sig#=%d, dire=%d, type=%d, pola=%d, debo=%d, data=%d}\n",
					i, (int)p_gpiocfg->signal_number, (int)p_gpiocfg->direction, (int)p_gpiocfg->type,
					(int)p_gpiocfg->polarity, (int)p_gpiocfg->debounce, (int)p_gpiocfg->data);
		}
	}
	printf("\n");

	return;
}


/********************************************************************************//*!
 * @fn  void sample_gpio_utility(void) 
 * @brief  Try calling the GPIO utility API. 
 * 
 * @details  Run the API of the subcategory "General-Purpose IO Utility Functions". 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/06  ---  #6192  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gpio_utility(void)
{
	uint32_t ui32result;

	/***********************************************************************/
	/* Usage: uint32_t alt_gpio_port_idcode_get(ALT_GPIO_PORT_t gpio_pid); */
	/***********************************************************************/
	printf("-- HWLib test: alt_gpio_port_idcode_get() --\n");
	ui32result = alt_gpio_port_idcode_get(ALT_GPIO_PORTA);
	printf("arg1=ALT_GPIO_PORTA(%d) --> result=0x%08X(%d)\n", (int)ALT_GPIO_PORTA, (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_idcode_get(ALT_GPIO_PORTB);
	printf("arg1=ALT_GPIO_PORTB(%d) --> result=0x%08X(%d)\n", (int)ALT_GPIO_PORTB, (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_idcode_get(ALT_GPIO_PORTC);
	printf("arg1=ALT_GPIO_PORTC(%d) --> result=0x%08X(%d)\n", (int)ALT_GPIO_PORTC, (int)ui32result, (int)ui32result);
	printf("\n");

	/********************************************************************/
	/* Usage: uint32_t alt_gpio_port_ver_get(ALT_GPIO_PORT_t gpio_pid);	*/
	/********************************************************************/
	printf("-- HWLib test : alt_gpio_port_ver_get() --\n");
	ui32result = alt_gpio_port_ver_get(ALT_GPIO_PORTA);
	printf("arg1=ALT_GPIO_PORTA(%d) --> result=0x%08X (%d)\n", (int)ALT_GPIO_PORTA, (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_ver_get(ALT_GPIO_PORTB);
	printf("arg1=ALT_GPIO_PORTB(%d) --> result=0x%08X (%d)\n", (int)ALT_GPIO_PORTB, (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_ver_get(ALT_GPIO_PORTC);
	printf("arg1=ALT_GPIO_PORTC(%d) --> result=0x%08X (%d)\n", (int)ALT_GPIO_PORTC, (int)ui32result, (int)ui32result);
	printf("\n");

	/************************************************************************/
	/* Usage: ALT_GPIO_PORT_t alt_gpio_bit_to_pid(ALT_GPIO_1BIT_t pin_num);	*/
	/************************************************************************/
	/********************************************************************************/
	/* Usage: ALT_GPIO_PORTBIT_t alt_gpio_bit_to_port_pin(ALT_GPIO_1BIT_t pin_num);	*/
	/********************************************************************************/
	printf("-- HWLib test : alt_gpio_bit_to_pid(), alt_gpio_bit_to_port_pin() --\n");
	printf("arg1=ALT_HLGPI_13(%d) --> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_13, (int)alt_gpio_bit_to_pid(ALT_HLGPI_13), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_13));
	printf("arg1=ALT_HLGPI_12(%d) --> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_12, (int)alt_gpio_bit_to_pid(ALT_HLGPI_12), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_12));
	printf("arg1=ALT_HLGPI_11(%d) --> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_11, (int)alt_gpio_bit_to_pid(ALT_HLGPI_11), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_11));
	printf("arg1=ALT_HLGPI_10(%d) --> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_10, (int)alt_gpio_bit_to_pid(ALT_HLGPI_10), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_10));
	printf("arg1=ALT_HLGPI_9 (%d)--> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_9, (int)alt_gpio_bit_to_pid(ALT_HLGPI_9), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_9));
	printf("arg1=ALT_HLGPI_8 (%d)--> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_8, (int)alt_gpio_bit_to_pid(ALT_HLGPI_8), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_8));
	printf("arg1=ALT_HLGPI_7 (%d)--> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_7, (int)alt_gpio_bit_to_pid(ALT_HLGPI_7), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_7));
	printf("arg1=ALT_HLGPI_6 (%d)--> result{pid=%d, bit=%d}\n",
			(int)ALT_HLGPI_6, (int)alt_gpio_bit_to_pid(ALT_HLGPI_6), (int)alt_gpio_bit_to_port_pin(ALT_HLGPI_6));
	printf("arg1=ALT_GPIO_1BIT_61(%d) --> result{pid=%d, bit=%d}\n",
			(int)ALT_GPIO_1BIT_61, (int)alt_gpio_bit_to_pid(ALT_GPIO_1BIT_61), (int)alt_gpio_bit_to_port_pin(ALT_GPIO_1BIT_61));
	printf("\n");

	/*******************************************************************************************/
	/* Usage: ALT_GPIO_1BIT_t alt_gpio_port_pin_to_bit(ALT_GPIO_PORT_t pid, uint32_t bitmask); */
	/*******************************************************************************************/
	printf("-- HWLib test : alt_gpio_port_pin_to_bit() --\n");
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT26);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT26 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT25);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT25 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT24);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT24 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT23);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT23 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT22);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT22 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT21);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT21 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT20);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT20 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT19);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT19 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	ui32result = alt_gpio_port_pin_to_bit(ALT_GPIO_PORTC, ALT_GPIO_BIT3);
	printf("arg1=ALT_GPIO_PORTC, arg2=ALT_GPIO_BIT3 --> result0x%08X(%d)\n", (int)ui32result, (int)ui32result);
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_print_usage(void) 
 * @brief  Display usage of the sample program. 
 * 
 * @details  Following information displays on the console.
 *            - Overview of HPS for push button functions (or operations)
 *            - Overview of HPS for DIP switch functions (or operations)
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/08/18  ---  #xxxxx  VK쐬. 
 **//*******************************************************************************/
void sample_test_print_usage(void)
{
	printf("+--- Usage ( Function of HPS-PUSHSW and HPS-DIPSW )  -----------+\n");
	printf("+ PUSHSW HPS 0(SW8) .... Issued GPIO interrupt, PUSHSW is detected at Rising-Edge.\n");
	printf("+ PUSHSW HPS 1(SW10) ... Issued GPIO interrupt at Falling-Edge.\n");
	printf("+ PUSHSW HPS 2(SW7) .... Issued GPIO interrupt at Rising-Edge.\n");
	printf("+ PUSHSW HPS 3(SW9) .... Issued GPIO interrupt at Falling-Edge.\n");
	printf("+ DIPSW HPS(SW14) 4 .... Exit Test loop!!! (DIPSW4=ON)\n");
	printf("+---------------------------------------------------------------+\n\n");
	
#if 0
	mFpɊ荞݃Rg[(GIC)̐ݒύXsA
	L̏Ŋ荞݂삵܂B
	@ - GPIO荞݌_@ sample_gpio_callback() N܂B
	@ - HPSpDIPSW4|[g͊荞݃gK[ƂȂB
	@ - HPSpPUSHSW4|[g͑SĊ荞݃gK[ƂB
	@@@PUSHSW-1(SW8) ... Rising-Edge
	@@@PUSHSW-2(SW10) .. Falling-Edge
	@@@PUSHSW-3(SW7) ... Rising-Edge
	@@@PUSHSW-4(SW9) ... Falling-Edge
#endif
	
	return;
}

/***********************************************************************************
 * end of file 
 ***********************************************************************************/
