/********************************************************************************//*!
 * @file  sample_gptimer.c 
 * @brief  To use HWLib for the General-Purpose Timer example program. 
 * 
 * @details  Run the API category below.
 *           - The General Purpose Timer Manager API  
 *               - Enable, Disable, and Status  
 *               - Counters Interface  
 *               - Interrupts  
 *               - Mode Control 
 * 
 * @note  nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 * @date 2014/08/18  ---  -----  v14.0 (AA̕\ǉ). 
 * 
 * @attention 
 * Copyright (C) 2013-2014 ALTIMA Corp./ELSENA,Inc. All Rights Reserved.\n
 *   This software is licensed "AS IS". 
 *   Please perform use of this software by a user's own responsibility and expense.
 *   It cannot guarantee in the maker side about the damage which occurred by the ab-
 *   ility not to use or use this software, and all damage that occurred secondarily.
 **//*******************************************************************************/

/***********************************************************************************
 *  includes 
 ***********************************************************************************/
#include <stdio.h>
#include <string.h>
#include "hwlib.h"
#include "alt_generalpurpose_io.h"
#include "alt_globaltmr.h"
#include "alt_clock_manager.h"
#include "alt_interrupt.h"
#include "alt_timers.h"
#include "alt_watchdog.h"

/***********************************************************************************
 *  externs 
 ***********************************************************************************/

/***********************************************************************************
 *  proto types 
 ***********************************************************************************/
static void sample_gptimer_print_specific_counter(ALT_GPT_TIMER_t tmr_id);
static void sample_gptimer_print_all_counter(void);
static void sample_gptimer_print_all_counter2(void);
static void sample_gptimer_print_specific_settings_status(ALT_GPT_TIMER_t tmr_id);
static void sample_gptimer_print_specific_settings(ALT_GPT_TIMER_t tmr_id);
static void sample_gptimer_print_all_settings(void);
static void sample_gptimer_print_all_settings2(void);
static void sample_gptimer_print_all_settings3(void);
static void sample_gptimer_start_all(void);
static void sample_gptimer_stop_all(void);
static void sample_gptimer_reset_all(void);
static void sample_gptimer_ienable_all(void);
static void sample_gptimer_idisable_all(void);
static void sample_gptimer_print_setting_gic(ALT_INT_INTERRUPT_t int_id);
static void sample_gptimer_print_all_int_settings(void);
void sample_test_init(void);
void sample_test_uninit(void);
void sample_gptimer_setting_gic(void);
void sample_gptimer_callback_OSC1_TMR0(uint32_t icciar, void *context);
void sample_gptimer_callback_OSC1_TMR1(uint32_t icciar, void *context);
void sample_gptimer_callback_SP_TMR0(uint32_t icciar, void *context);
void sample_gptimer_callback_SP_TMR1(uint32_t icciar, void *context);
static void sample_gptimer_changemode_and_reset(ALT_GPT_TIMER_t tmr_id, ALT_GPT_RESTART_MODE_t mode);
void sample_test_print_usage(void);
void sample_test_main(void);
// prototype (Common Functions) //
static void sample_gpio_config_helioboard(void);

/***********************************************************************************
 *  definitions (define, enum, typedef, etc..) 
 ***********************************************************************************/
/** memo: port asign ***************/
/* Signal Name <--> HPS Port Name  */
/* USER_DIPSW_HPS0 <--> HPS_GPI13  */
/* USER_DIPSW_HPS1 <--> HPS_GPI12  */
/* USER_DIPSW_HPS2 <--> HPS_GPI11  */
/* USER_DIPSW_HPS3 <--> HPS_GPI10  */
/* USER_PB_HPS0 <--> HPS_GPI9      */
/* USER_PB_HPS1 <--> HPS_GPI8      */
/* USER_PB_HPS2 <--> HPS_GPI7      */
/* USER_PB_HPS3 <--> HPS_GPI6      */
/* USER_HPS_GPIO61 <--> HPS_GPIO61 */
/***********************************/
#define ALT_GPIO_PORTBIT_DIPS1		(ALT_GPIO_BIT26)
#define ALT_GPIO_PORTBIT_DIPS2		(ALT_GPIO_BIT25)
#define ALT_GPIO_PORTBIT_DIPS3		(ALT_GPIO_BIT24)
#define ALT_GPIO_PORTBIT_DIPS4		(ALT_GPIO_BIT23)
#define ALT_GPIO_PORTBIT_DIPSWALL	((ALT_GPIO_BIT26)|(ALT_GPIO_BIT25)|(ALT_GPIO_BIT24)|(ALT_GPIO_BIT23))
#define ALT_GPIO_LSBBIT_DIPSWS		(23)
#define ALT_GPIO_PORTBIT_PUSH2		(ALT_GPIO_BIT22)
#define ALT_GPIO_PORTBIT_PUSH0		(ALT_GPIO_BIT21)
#define ALT_GPIO_PORTBIT_PUSH3		(ALT_GPIO_BIT20)
#define ALT_GPIO_PORTBIT_PUSH1		(ALT_GPIO_BIT19)
#define ALT_GPIO_PORTBIT_PUSHSWALL	((ALT_GPIO_BIT22)|(ALT_GPIO_BIT21)|(ALT_GPIO_BIT20)|(ALT_GPIO_BIT19))
#define ALT_GPIO_PORTBIT_SWALL		((ALT_GPIO_PORTBIT_DIPSWALL)|(ALT_GPIO_PORTBIT_PUSHSWALL))

/***********************************************************************************
 *  variables 
 ***********************************************************************************/

/***********************************************************************************
 *  functions 
 ***********************************************************************************/

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_specific_counter(ALT_GPT_TIMER_t tmr_id) 
 * @brief  Display the value of specific general-purpose timer count registers. 
 * 
 * @details  Displays the register value that you can get with the following APIs.
 *           - alt_gpt_counter_get(). 
 *           - alt_gpt_reset_value_get(). 
 *           - alt_gpt_maxcounter_get(). 
 *           - alt_gpt_curtime_millisecs_get(). 
 *           - alt_gpt_time_millisecs_get(). 
 *           - alt_gpt_maxtime_millisecs_get(). 
 * @param[in] tmr_id  general-purpose timer identification types. 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_specific_counter(ALT_GPT_TIMER_t tmr_id)
{
	printf("CounterValue[cur/ini/max]= 0x%08X / 0x%08X / 0x%08X\n",
			(int)alt_gpt_counter_get(tmr_id),
			(int)alt_gpt_reset_value_get(tmr_id),
			(int)alt_gpt_maxcounter_get(tmr_id));
	printf("MilliSeconds[cur/ini/max]= %d / %d /%d\n",
			(int)alt_gpt_curtime_millisecs_get(tmr_id),
			(int)alt_gpt_time_millisecs_get(tmr_id),
			(int)alt_gpt_maxtime_millisecs_get(tmr_id));

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_all_counter(void) 
 * @brief  Display the value of all timer counters. 
 * 
 * @details  Call sample_gptimer_print_specific_counter for all timers (including 
 *           the WDT, Global Timer).
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_all_counter(void)
{
	printf("<< ALT_GPT_CPU_GLOBAL_TMR >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_CPU_GLOBAL_TMR);
	printf("\n");

	printf("<< ALT_GPT_CPU_PRIVATE_TMR >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_CPU_PRIVATE_TMR);
	printf("\n");

	printf("<< ALT_GPT_CPU_WDTGPT_TMR >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_CPU_WDTGPT_TMR);
	printf("\n");

	printf("<< ALT_GPT_OSC1_TMR0 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR0);
	printf("\n");

	printf("<< ALT_GPT_OSC1_TMR1 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR1);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR0 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR0);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR1 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR1);
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_all_counter2(void) 
 * @brief  Display the value of all general-purpose timer counters. 
 * 
 * @details  Call sample_gptimer_print_specific_counter for the general-purpose 
 *           timers of all (except for the WDT, Global Timer).
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_all_counter2(void)
{
	printf("<< ALT_GPT_OSC1_TMR0 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR0);
	printf("\n");

	printf("<< ALT_GPT_OSC1_TMR1 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR1);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR0 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR0);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR1 >>\n");
	sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR1);
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_specific_settings_status(ALT_GPT_TIMER_t tmr_id) 
 * @brief  Display the value of specific general-purpose timer setting status. 
 * 
 * @details  Displays timer setting status that you can get with the following APIs.
 *           - alt_gpt_mode_get(). 
 *           - alt_gpt_tmr_is_running(). 
 *           - alt_gpt_int_is_enabled(). 
 *           - alt_gpt_int_is_pending(). 
 * @param[in] tmr_id  general-purpose timer identification types. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_specific_settings_status(ALT_GPT_TIMER_t tmr_id)
{
	/* Usage:  int32_t alt_gpt_mode_get(ALT_GPT_TIMER_t tmr_id);	*/
	printf("alt_gpt_mode_get() ............... result=%d\n", (int)alt_gpt_mode_get(tmr_id));
	/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_is_running(ALT_GPT_TIMER_t tmr_id);	*/
	printf("alt_gpt_tmr_is_running() ......... result=%d\n", (int)alt_gpt_tmr_is_running(tmr_id));
	/* Usage:  bool alt_gpt_int_is_enabled(ALT_GPT_TIMER_t tmr_id);	*/
	printf("alt_gpt_int_is_enabled() ......... result=%d\n", (int)alt_gpt_int_is_enabled(tmr_id));
	/* Usage:  ALT_STATUS_CODE alt_gpt_int_is_pending(ALT_GPT_TIMER_t tmr_id);	*/
	printf("alt_gpt_int_is_pending() ......... result=%d\n", (int)alt_gpt_int_is_pending(tmr_id));

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_specific_settings(ALT_GPT_TIMER_t tmr_id) 
 * @brief  Display the value of specific general-purpose timer settings. 
 * 
 * @details  Displays timer settings that you can get with the various APIs.
 * @param[in] tmr_id  general-purpose timer identification types. 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_specific_settings(ALT_GPT_TIMER_t tmr_id)
{
	uint32_t	ui32val;

	sample_gptimer_print_specific_settings_status(tmr_id);

	/* Usage:  uint32_t alt_gpt_prescaler_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_prescaler_get(tmr_id);
	printf("alt_gpt_prescaler_get() .......... result=%d\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_freq_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_freq_get(tmr_id);
	printf("alt_gpt_freq_get() ............... result=%d[Hz]\n", (int)ui32val);

	/* Usage:  uint32_t alt_gpt_counter_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_counter_get(tmr_id);
	printf("alt_gpt_counter_get() ............ result=0x%08X(%d)\n", (int)ui32val, (int)ui32val);
	/* Usage:  uint32_t alt_gpt_reset_value_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_reset_value_get(tmr_id);
	printf("alt_gpt_reset_value_get() ........ result=0x%08X(%d)\n", (int)ui32val, (int)ui32val);
	/* Usage:  uint32_t alt_gpt_maxcounter_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_maxcounter_get(tmr_id);
	printf("alt_gpt_maxcounter_get() ......... result=0x%08X(%d)\n", (int)ui32val, (int)ui32val);

	/* Usage:  uint32_t alt_gpt_time_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_time_get(tmr_id);
	printf("alt_gpt_time_get() ............... result=%d[period@sec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_time_millisecs_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_time_millisecs_get(tmr_id);
	printf("alt_gpt_time_millisecs_get() ..... result=%d[period@msec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_time_microsecs_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_time_microsecs_get(tmr_id);
	printf("alt_gpt_time_microsecs_get() ..... result=%d[period@usec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_curtime_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_curtime_get(tmr_id);
	printf("alt_gpt_curtime_get() ............ result=%d[sec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_curtime_millisecs_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_curtime_millisecs_get(tmr_id);
	printf("alt_gpt_curtime_millisecs_get() .. result=%d[msec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_curtime_microsecs_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_curtime_microsecs_get(tmr_id);
	printf("alt_gpt_curtime_microsecs_get() .. result=%d[usec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_curtime_nanosecs_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_curtime_nanosecs_get(tmr_id);
	printf("alt_gpt_curtime_nanosecs_get() ... result=%d[nsec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_maxtime_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_maxtime_get(tmr_id);
	printf("alt_gpt_maxtime_get() ............ result=%d[period@sec]\n", (int)ui32val);
	/* Usage:  uint32_t alt_gpt_maxtime_millisecs_get(ALT_GPT_TIMER_t tmr_id);	*/
	ui32val = alt_gpt_maxtime_millisecs_get(tmr_id);
	printf("alt_gpt_maxtime_millisecs_get() .. result=%d[period@msec]\n", (int)ui32val);

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_all_settings(void) 
 * @brief  Display the value of all timer settings. 
 * 
 * @details  Call sample_gptimer_print_specific_settings for all timers (including 
 *           the WDT, Global Timer).
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_all_settings(void)
{
	printf("<< ALT_GPT_CPU_GLOBAL_TMR >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_CPU_GLOBAL_TMR);
	printf("\n");

	printf("<< ALT_GPT_CPU_PRIVATE_TMR >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_CPU_PRIVATE_TMR);
	printf("\n");

	printf("<< ALT_GPT_CPU_WDTGPT_TMR >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_CPU_WDTGPT_TMR);
	printf("\n");

	printf("<< ALT_GPT_OSC1_TMR0 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_OSC1_TMR0);
	printf("\n");

	printf("<< ALT_GPT_OSC1_TMR1 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_OSC1_TMR1);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR0 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_SP_TMR0);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR1 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_SP_TMR1);
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_all_settings2(void) 
 * @brief  Display the value of all general-purpose timer settings. 
 * 
 * @details  Call sample_gptimer_print_specific_settings for the general-purpose 
 *           timers of all (except for the WDT, Global Timer).
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_all_settings2(void)
{
	printf("<< ALT_GPT_OSC1_TMR0 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_OSC1_TMR0);
	printf("\n");

	printf("<< ALT_GPT_OSC1_TMR1 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_OSC1_TMR1);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR0 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_SP_TMR0);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR1 >>\n");
	sample_gptimer_print_specific_settings(ALT_GPT_SP_TMR1);
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_all_settings3(void) 
 * @brief  Display the value of all general-purpose timer setting status. 
 * 
 * @details  Call sample_gptimer_print_specific_settings_status for the general- 
 *           purpose timers of all (except for the WDT, Global Timer).
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_all_settings3(void)
{
	printf("<< ALT_GPT_OSC1_TMR0 >>\n");
	sample_gptimer_print_specific_settings_status(ALT_GPT_OSC1_TMR0);
	printf("\n");

	printf("<< ALT_GPT_OSC1_TMR1 >>\n");
	sample_gptimer_print_specific_settings_status(ALT_GPT_OSC1_TMR1);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR0 >>\n");
	sample_gptimer_print_specific_settings_status(ALT_GPT_SP_TMR0);
	printf("\n");

	printf("<< ALT_GPT_SP_TMR1 >>\n");
	sample_gptimer_print_specific_settings_status(ALT_GPT_SP_TMR1);
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_start_all(void) 
 * @brief  Start the timers for the general-purpose timers.
 * 
 * @details  Trial related to the general-purpose timer start APIs.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_start_all(void)
{
	ALT_STATUS_CODE	result_code;

	/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_start(ALT_GPT_TIMER_t tmr_id);	*/
	printf("-- HWLib test : alt_gpt_tmr_start() --\n");
	printf(" - ALT_GPT_OSC1_TMR0 timer start!!\n");
	result_code = alt_gpt_tmr_start(ALT_GPT_OSC1_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_start(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_OSC1_TMR1 timer start!!\n");
	result_code = alt_gpt_tmr_start(ALT_GPT_OSC1_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_start(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR0 timer start!!\n");
	result_code = alt_gpt_tmr_start(ALT_GPT_SP_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_start(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR1 timer start!!\n");
	result_code = alt_gpt_tmr_start(ALT_GPT_SP_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_start(); result=%d\n", (int)result_code);
	}
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_stop_all(void) 
 * @brief  Stop the timers for the general-purpose timers.
 * 
 * @details  Trial related to the general-purpose timer stop APIs.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_stop_all(void)
{
	ALT_STATUS_CODE	result_code;

	/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_stop(ALT_GPT_TIMER_t tmr_id);	*/
	printf("-- HWLib test : alt_gpt_tmr_stop() --\n");
	printf(" - ALT_GPT_OSC1_TMR0 timer stop!!\n");
	result_code = alt_gpt_tmr_stop(ALT_GPT_OSC1_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_stop(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_OSC1_TMR1 timer stop!!\n");
	result_code = alt_gpt_tmr_stop(ALT_GPT_OSC1_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_stop(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR0 timer stop!!\n");
	result_code = alt_gpt_tmr_stop(ALT_GPT_SP_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_stop(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR1 timer stop!!\n");
	result_code = alt_gpt_tmr_stop(ALT_GPT_SP_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_stop(); result=%d\n", (int)result_code);
	}
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_reset_all(void) 
 * @brief  Reset the timers for the general-purpose timers.
 * 
 * @details  Trial related to the general-purpose timer reset APIs.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_reset_all(void)
{
	ALT_STATUS_CODE	result_code;

	/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_reset(ALT_GPT_TIMER_t tmr_id);	*/
	printf("-- HWLib test : alt_gpt_tmr_reset() --\n");
	printf(" - ALT_GPT_OSC1_TMR0 timer reset!!\n");
	result_code = alt_gpt_tmr_reset(ALT_GPT_OSC1_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_reset(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_OSC1_TMR1 timer reset!!\n");
	result_code = alt_gpt_tmr_reset(ALT_GPT_OSC1_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_reset(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR0 timer reset!!\n");
	result_code = alt_gpt_tmr_reset(ALT_GPT_SP_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_reset(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR1 timer reset!!\n");
	result_code = alt_gpt_tmr_reset(ALT_GPT_SP_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_reset(); result=%d\n", (int)result_code);
	}
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_ienable_all(void) 
 * @brief  Enabling interrupts of all general-purpose timers. 
 * 
 * @details  Trial related to the interrupt enable APIs.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_ienable_all(void)
{
	ALT_STATUS_CODE	result_code;

	/* Usage:  ALT_STATUS_CODE alt_gpt_int_if_pending_clear(ALT_GPT_TIMER_t tmr_id);	*/
	printf("-- HWLib test : alt_gpt_int_if_pending_clear() --\n");
	printf("alt_gpt_int_if_pending_clear(ALT_GPT_OSC1_TMR0); result=%d\n", (int)alt_gpt_int_if_pending_clear(ALT_GPT_OSC1_TMR0));
	printf("alt_gpt_int_if_pending_clear(ALT_GPT_OSC1_TMR1); result=%d\n", (int)alt_gpt_int_if_pending_clear(ALT_GPT_OSC1_TMR1));
	printf("alt_gpt_int_if_pending_clear(ALT_GPT_SP_TMR0); result=%d\n", (int)alt_gpt_int_if_pending_clear(ALT_GPT_SP_TMR0));
	printf("alt_gpt_int_if_pending_clear(ALT_GPT_SP_TMR1); result=%d\n", (int)alt_gpt_int_if_pending_clear(ALT_GPT_SP_TMR1));
	printf("\n");

	/* Usage: ALT_STATUS_CODE alt_int_dist_enable(ALT_INT_INTERRUPT_t int_id);	*/
	result_code = alt_int_dist_enable(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_enable(TIMER_OSC1_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_enable(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_enable(TIMER_OSC1_1_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_enable(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_enable(TIMER_L4SP_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_enable(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_enable(TIMER_L4SP_1_IRQ) result=%d\n", (int)result_code);
	}

	/* Usage: ALT_STATUS_CODE alt_int_dist_secure_enable(ALT_INT_INTERRUPT_t int_id);	*/
	result_code = alt_int_dist_secure_enable(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_secure_enable(TIMER_OSC1_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_secure_enable(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_secure_enable(TIMER_OSC1_1_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_secure_enable(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_secure_enable(TIMER_L4SP_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_secure_enable(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_secure_enable(TIMER_L4SP_1_IRQ) result=%d\n", (int)result_code);
	}

	/* Usage:  ALT_STATUS_CODE alt_gpt_int_enable(ALT_GPT_TIMER_t tmr_id);	*/
	printf("-- HWLib test : alt_gpt_int_enable() --\n");
	printf(" - ALT_GPT_OSC1_TMR0 timer interrupt enable!!\n");
	result_code = alt_gpt_int_enable(ALT_GPT_OSC1_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_enable(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_OSC1_TMR1 timer interrupt enable!!\n");
	result_code = alt_gpt_int_enable(ALT_GPT_OSC1_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_enable(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR0 timer interrupt enable!!\n");
	result_code = alt_gpt_int_enable(ALT_GPT_SP_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_enable(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR1 timer interrupt enable!!\n");
	result_code = alt_gpt_int_enable(ALT_GPT_SP_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_enable(); result=%d\n", (int)result_code);
	}
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_idisable_all(void) 
 * @brief  Disabling interrupts of all general-purpose timers. 
 * 
 * @details  Trial related to the interrupt disable APIs. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_idisable_all(void)
{
	ALT_STATUS_CODE	result_code;

	/* Usage:  ALT_STATUS_CODE alt_gpt_int_disable(ALT_GPT_TIMER_t tmr_id);	*/
	printf("-- HWLib test : alt_gpt_int_disable() --\n");
	printf(" - ALT_GPT_OSC1_TMR0 timer interrupt disable!!\n");
	result_code = alt_gpt_int_disable(ALT_GPT_OSC1_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_disable(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_OSC1_TMR1 timer interrupt disable!!\n");
	result_code = alt_gpt_int_disable(ALT_GPT_OSC1_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_disable(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR0 timer interrupt disable!!\n");
	result_code = alt_gpt_int_disable(ALT_GPT_SP_TMR0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_disable(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR1 timer interrupt disable!!\n");
	result_code = alt_gpt_int_disable(ALT_GPT_SP_TMR1);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_int_disable(); result=%d\n", (int)result_code);
	}
	printf("\n");


	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_init(void) 
 * @brief  Initializing the sample program. 
 * 
 * @details  Call the initialization function for the general-purpose timer API. 
 *           Other, do the following.
 *           -# Configure the Helio Board GPIO settings. 
 *           -# Setting of general-purpose timers (Use HWLib). 
 *           -# Setting of GIC (General-Purpose timer interrupt). 
 *           -# Print general-purpose timer settings. 
 *           -# Start the timers for the general-purpose timers. 
 *           -# Enabling interrupts of all general-purpose timers.  
 *           -# Print GIC settings of all general-purpose timer interrupts. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_test_init(void)
{
	ALT_STATUS_CODE result_code;

	/* Usage:  ALT_STATUS_CODE alt_gpt_mode_set(ALT_GPT_TIMER_t tmr_id, ALT_GPT_RESTART_MODE_t mode);	*/
	/* Usage:  ALT_STATUS_CODE alt_gpt_counter_set(ALT_GPT_TIMER_t tmr_id, uint32_t val);	*/
	/* Usage:  ALT_STATUS_CODE alt_gpt_prescaler_set(ALT_GPT_TIMER_t tmr_id, uint32_t val);	*/

	sample_gpio_config_helioboard();

	/* Usage:  ALT_STATUS_CODE alt_gpt_all_tmr_init(void);	*/
	printf("-- HWLib test : alt_gpt_all_tmr_init() --\n");
	result_code = alt_gpt_all_tmr_init();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_all_tmr_init() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");

	printf("======== Print Initial GPTimer Settings ========\n");
	sample_gptimer_print_all_settings();

	// GPT Timer Stop & Interrupt Disable
	sample_gptimer_stop_all();
	sample_gptimer_idisable_all();

	/* Usage:  ALT_STATUS_CODE alt_gpt_prescaler_set(ALT_GPT_TIMER_t tmr_id, uint32_t val);	*/
	/* ##	Sets the clock prescaler value of the specified timer. Valid for CPU_PRIVATE_TMR and
	 * 		CPU_GLOBAL_TMR. Returns an error if called with a tmr_id of OSC1_TMR0, OSC1_TMR1,
	 * 		SP_TMR0, or SP_TMR1 since they have no prescaler.	*/
	/* 		--> Do Test using arg1 of ALT_GPT_CPU_WDTGPT_TMR. */
	printf("-- HWLib test : alt_gpt_prescaler_set(ALT_GPT_CPU_WDTGPT_TMR, 0) --\n");
	result_code = alt_gpt_prescaler_set(ALT_GPT_CPU_WDTGPT_TMR , 0);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_prescaler_set(); result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");

	/* Usage:  ALT_STATUS_CODE alt_gpt_counter_set(ALT_GPT_TIMER_t tmr_id, uint32_t val);	*/
	printf("-- HWLib test : alt_gpt_counter_set() --\n");
	printf(" - ALT_GPT_OSC1_TMR0 counter set to 500,000,000 @25MHz (20sec)\n");
	result_code = alt_gpt_counter_set(ALT_GPT_OSC1_TMR0, 500000000);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_counter_set(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_OSC1_TMR1 counter set to 1,000,000,000 @25MHz (40sec)\n");
	result_code = alt_gpt_counter_set(ALT_GPT_OSC1_TMR1, 1000000000);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_counter_set(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR0 counter set to 500,000,000 @100MHz (5sec)\n");
	result_code = alt_gpt_counter_set(ALT_GPT_SP_TMR0, 500000000);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_counter_set(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR1 counter set to 1,000,000,000 @100MHz (10sec)\n");
	result_code = alt_gpt_counter_set(ALT_GPT_SP_TMR1, 1000000000);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_counter_set(); result=%d\n", (int)result_code);
	}
	printf("\n");

	/* Usage:  ALT_STATUS_CODE alt_gpt_mode_set(ALT_GPT_TIMER_t tmr_id, ALT_GPT_RESTART_MODE_t mode);	*/
	printf("-- HWLib test : alt_gpt_mode_set() --\n");
	printf(" - ALT_GPT_OSC1_TMR0 response mode set to ALT_GPT_RESTART_MODE_PERIODIC\n");
	result_code = alt_gpt_mode_set(ALT_GPT_OSC1_TMR0, ALT_GPT_RESTART_MODE_PERIODIC);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_mode_set(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_OSC1_TMR1 response mode set to ALT_GPT_RESTART_MODE_PERIODIC\n");
	result_code = alt_gpt_mode_set(ALT_GPT_OSC1_TMR1, ALT_GPT_RESTART_MODE_PERIODIC);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_mode_set(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR0 response mode set to ALT_GPT_RESTART_MODE_PERIODIC\n");
	result_code = alt_gpt_mode_set(ALT_GPT_SP_TMR0, ALT_GPT_RESTART_MODE_PERIODIC);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_mode_set(); result=%d\n", (int)result_code);
	}
	printf(" - ALT_GPT_SP_TMR1 response mode set to ALT_GPT_RESTART_MODE_PERIODIC\n");
	result_code = alt_gpt_mode_set(ALT_GPT_SP_TMR1, ALT_GPT_RESTART_MODE_PERIODIC);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_mode_set(); result=%d\n", (int)result_code);
	}
	printf("\n");


	// Interrupt GIC Config
	sample_gptimer_setting_gic();

	printf("======== Print Test GPTimer Settings ========\n");
	sample_gptimer_print_all_settings();

	printf("--- All GPT Start!!! ---\n");
	sample_gptimer_start_all();
	sample_gptimer_print_all_settings3();

	printf("--- All GPT Interrupts Enable!!! ---\n");
	sample_gptimer_ienable_all();
	sample_gptimer_print_all_settings3();

	printf("======== Print Test Interrupt Settings ========\n");
	sample_gptimer_print_all_int_settings();


	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_uninit(void) 
 * @brief  Uninitializing the sample program. 
 * 
 * @details  Call the uninitialization function for the general-purpose timer API.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_test_uninit(void)
{
	ALT_STATUS_CODE result_code;

	/* Usage:  ALT_STATUS_CODE alt_gpt_all_tmr_uninit(void);	*/
	printf("-- HWLib test : alt_gpt_all_tmr_uninit() --\n");
	result_code = alt_gpt_all_tmr_uninit();
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_all_tmr_uninit() result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");


	return;
}


/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_setting_gic(ALT_INT_INTERRUPT_t int_id) 
 * @brief  Print GIC settings of specific general-purpose timer interrupts.
 * 
 * @details  Displays the specific interrupt settings of detail.
 * @param[in] int_id  interrupt identification types.  
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_setting_gic(ALT_INT_INTERRUPT_t int_id)
{
	uint32_t ui32temp = 0;
	ALT_INT_TRIGGER_t trigger_type = ALT_INT_TRIGGER_NA;

	//// Print initial value
	///****************************************************************************************/
	///* Usage: ALT_STATUS_CODE alt_int_dist_is_secure(ALT_INT_INTERRUPT_t int_id);	*/
	///* Usage: ALT_STATUS_CODE alt_int_dist_is_enabled(ALT_INT_INTERRUPT_t int_id);	*/
	///* Usage: ALT_STATUS_CODE alt_int_dist_is_pending(ALT_INT_INTERRUPT_t int_id);	*/
	///* Usage: ALT_STATUS_CODE alt_int_dist_is_active(ALT_INT_INTERRUPT_t int_id);	*/
	///* Usage: ALT_STATUS_CODE alt_int_dist_priority_get(ALT_INT_INTERRUPT_t int_id, uint32_t* priority);	*/
	///* Usage: ALT_STATUS_CODE alt_int_dist_target_get(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t* target);	*/
	///* Usage: ALT_STATUS_CODE alt_int_dist_trigger_get(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t* trigger_type);	*/
	///****************************************************************************************/
	printf("alt_int_dist_is_secure() ...... result=%d\n", (int)alt_int_dist_is_secure(int_id));
	printf("alt_int_dist_is_enabled() ..... result=%d\n", (int)alt_int_dist_is_enabled(int_id));
	printf("alt_int_dist_is_pending() ..... result=%d\n", (int)alt_int_dist_is_pending(int_id));
	printf("alt_int_dist_is_active() ...... result=%d\n", (int)alt_int_dist_is_active(int_id));
	printf("alt_int_dist_priority_get() ... result=%d out=0x%08X\n", (int)alt_int_dist_priority_get(int_id, &ui32temp), (int)ui32temp);
	printf("alt_int_dist_target_get() ..... result=%d out=0x%08X\n", (int)alt_int_dist_target_get(int_id, &ui32temp), (int)ui32temp);
	printf("alt_int_dist_trigger_get() .... result=%d out=%d\n", (int)alt_int_dist_trigger_get(int_id, &trigger_type), (int)trigger_type);

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_print_all_int_settings(void) 
 * @brief  Print GIC settings of all general-purpose timer interrupts. 
 * 
 * @details  Displays the three types of interrupt settings.
 *           - ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ 
 *           - ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ 
 *           - ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ 
 *           - ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_print_all_int_settings(void)
{
	printf("<< ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ >>\n");
	sample_gptimer_print_setting_gic(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ);
	printf("\n");

	printf("<< ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ >>\n");
	sample_gptimer_print_setting_gic(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ);
	printf("\n");

	printf("<< ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ >>\n");
	sample_gptimer_print_setting_gic(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ);
	printf("\n");

	printf("<< ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ >>\n");
	sample_gptimer_print_setting_gic(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ);
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_gptimer_setting_gic(void) 
 * @brief  Setting of GIC (General-Purpose timer interrupt).
 * 
 * @details    In order to receive a general-purpose timer interrupt setting of GIC.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gptimer_setting_gic(void)
{
	ALT_STATUS_CODE result_code;


	/* Usage: ALT_STATUS_CODE alt_int_isr_register(ALT_INT_INTERRUPT_t int_id, alt_int_callback_t callback, void* context);	*/
	/*  typedef void(* alt_int_callback_t)(uint32_t icciar, void *context) */
	result_code = alt_int_isr_register(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ, sample_gptimer_callback_OSC1_TMR0, NULL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_isr_register(TIMER_OSC1_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_isr_register(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ, sample_gptimer_callback_OSC1_TMR1, NULL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_isr_register(TIMER_OSC1_1_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_isr_register(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ, sample_gptimer_callback_SP_TMR0, NULL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_isr_register(TIMER_L4SP_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_isr_register(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ, sample_gptimer_callback_SP_TMR1, NULL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_isr_register(TIMER_L4SP_1_IRQ) result=%d\n", (int)result_code);
	}

	/* Usage: ALT_STATUS_CODE alt_int_dist_target_set(ALT_INT_INTERRUPT_t int_id, alt_int_cpu_target_t target);	*/
	result_code = alt_int_dist_target_set(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ, 1);		// arg2=targetCPU (Only Core#0)
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_target_set(TIMER_OSC1_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_target_set(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ, 1);		// arg2=targetCPU (Only Core#0)
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_target_set(TIMER_OSC1_1_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_target_set(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ, 1);		// arg2=targetCPU (Only Core#0)
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_target_set(TIMER_L4SP_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_target_set(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ, 1);		// arg2=targetCPU (Only Core#0)
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_int_dist_target_set(TIMER_L4SP_1_IRQ) result=%d\n", (int)result_code);
	}

	/* Usage: ALT_STATUS_CODE alt_int_dist_trigger_set(ALT_INT_INTERRUPT_t int_id, ALT_INT_TRIGGER_t trigger_type);	*/
	result_code = alt_int_dist_trigger_set(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ, ALT_INT_TRIGGER_LEVEL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_trigger_set(TIMER_OSC1_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_trigger_set(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ, ALT_INT_TRIGGER_LEVEL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_trigger_set(TIMER_OSC1_1_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_trigger_set(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ, ALT_INT_TRIGGER_LEVEL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_trigger_set(TIMER_L4SP_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_trigger_set(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ, ALT_INT_TRIGGER_LEVEL);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_trigger_set(TIMER_L4SP_1_IRQ) result=%d\n", (int)result_code);
	}

	/* Usage: ALT_STATUS_CODE alt_int_dist_priority_set(ALT_INT_INTERRUPT_t int_id, uint32_t priority);	*/
	result_code = alt_int_dist_priority_set(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ, 99);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_priority_set(TIMER_OSC1_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_priority_set(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ, 99);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_priority_set(TIMER_OSC1_1_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_priority_set(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ, 99);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_priority_set(TIMER_L4SP_0_IRQ) result=%d\n", (int)result_code);
	}
	result_code = alt_int_dist_priority_set(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ, 99);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_int_dist_priority_set(TIMER_L4SP_1_IRQ) result=%d\n", (int)result_code);
	}

	return;
}

/********************************************************************************//*!
 * @fn  void sample_gptimer_callback_OSC1_TMR0(uint32_t icciar, void *context) 
 * @brief  L4_OSC1 Timer0 (ALT_GPT_OSC1_TMR0) interrupt callback function.
 * 
 * @details  working opportunity an interrupt from the ALT_GPT_OSC1_TMR0, indicate 
 *           an interrupt, and the interrupt pending bit clear.
 * @param[in] icciar  The callback to use when the given interrupt is issued. 
 * @param[in] context  The user provided context.  
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gptimer_callback_OSC1_TMR0(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;

	// WDT Pending Clear
	/* Usage:  ALT_STATUS_CODE alt_gpt_int_clear_pending(ALT_GPT_TIMER_t tmr_id);	*/
	result_code = alt_gpt_int_clear_pending(ALT_GPT_OSC1_TMR0);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT1]ERROR!!: alt_gpt_int_clear_pending(ALT_GPT_OSC1_TMR0) result=%d\n", (int)result_code);
	}

	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT1]ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
	}

	printf("[INTERRUPT1] ALT_INT_INTERRUPT_TIMER_OSC1_0_IRQ is occurred!!\n");
	//sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR0);

	return;
}

/********************************************************************************//*!
 * @fn  void sample_gptimer_callback_OSC1_TMR1(uint32_t icciar, void *context) 
 * @brief  L4_OSC1 Timer1 (ALT_GPT_OSC1_TMR1) interrupt callback function.
 * 
 * @details  working opportunity an interrupt from the ALT_GPT_OSC1_TMR1, indicate 
 *           an interrupt, and the interrupt pending bit clear.
 * @param[in] icciar  The callback to use when the given interrupt is issued. 
 * @param[in] context  The user provided context.  
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gptimer_callback_OSC1_TMR1(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;

	// WDT Pending Clear
	/* Usage:  ALT_STATUS_CODE alt_gpt_int_clear_pending(ALT_GPT_TIMER_t tmr_id);	*/
	result_code = alt_gpt_int_clear_pending(ALT_GPT_OSC1_TMR1);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT2]ERROR!!: alt_gpt_int_clear_pending(ALT_GPT_OSC1_TMR1) result=%d\n", (int)result_code);
	}

	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT2]ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
	}

	printf("[INTERRUPT2] ALT_INT_INTERRUPT_TIMER_OSC1_1_IRQ is occurred!!\n");
	//sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR1);

	return;
}

/********************************************************************************//*!
 * @fn  void sample_gptimer_callback_SP_TMR0(uint32_t icciar, void *context) 
 * @brief  L4_SP Timer0 (ALT_GPT_SP_TMR0) interrupt callback function.
 * 
 * @details  working opportunity an interrupt from the ALT_GPT_SP_TMR0, indicate 
 *           an interrupt, and the interrupt pending bit clear.
 * @param[in] icciar  The callback to use when the given interrupt is issued. 
 * @param[in] context  The user provided context.  
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gptimer_callback_SP_TMR0(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;

	// WDT Pending Clear
	/* Usage:  ALT_STATUS_CODE alt_gpt_int_clear_pending(ALT_GPT_TIMER_t tmr_id);	*/
	result_code = alt_gpt_int_clear_pending(ALT_GPT_SP_TMR0);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT3]ERROR!!: alt_gpt_int_clear_pending(ALT_GPT_SP_TMR0) result=%d\n", (int)result_code);
	}

	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT3]ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
	}

	printf("[INTERRUPT3] ALT_INT_INTERRUPT_TIMER_L4SP_0_IRQ is occurred!!\n");
	//sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR0);

	return;
}

/********************************************************************************//*!
 * @fn  void sample_gptimer_callback_SP_TMR1(uint32_t icciar, void *context) 
 * @brief  L4_SP Timer1 (ALT_GPT_SP_TMR1) interrupt callback function.
 * 
 * @details  working opportunity an interrupt from the ALT_GPT_SP_TMR1, indicate 
 *           an interrupt, and the interrupt pending bit clear.
 * @param[in] icciar  The callback to use when the given interrupt is issued. 
 * @param[in] context  The user provided context.  
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_gptimer_callback_SP_TMR1(uint32_t icciar, void *context)
{
	ALT_STATUS_CODE result_code;

	// WDT Pending Clear
	/* Usage:  ALT_STATUS_CODE alt_gpt_int_clear_pending(ALT_GPT_TIMER_t tmr_id);	*/
	result_code = alt_gpt_int_clear_pending(ALT_GPT_SP_TMR1);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT4]ERROR!!: alt_gpt_int_clear_pending(ALT_GPT_SP_TMR1) result=%d\n", (int)result_code);
	}

	// GIC Pending Clear
	result_code = alt_int_dist_pending_clear(ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("[INTERRUPT4]ERROR!!: alt_int_dist_pending_clear() result=%d\n", (int)result_code);
	}

	printf("[INTERRUPT4] ALT_INT_INTERRUPT_TIMER_L4SP_1_IRQ is occurred!!\n");
	//sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR1);

	return;
}

/********************************************************************************//*!
 * @fn  static void sample_gptimer_changemode_and_reset(ALT_GPT_TIMER_t tmr_id, ALT_GPT_RESTART_MODE_t mode) 
 * @brief  Change mode and reset the specific general-purpose timer. 
 * 
 * @details  Change the specified general-purpose timer response mode. 
 *           Then reset the timer. 
 * @param[in] tmr_id  general-purpose timer identification types. 
 * @param[in] mode  ALT_GPT_RESTART_MODE_PERIODIC / ALT_GPT_RESTART_MODE_ONESHOT. 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gptimer_changemode_and_reset(ALT_GPT_TIMER_t tmr_id, ALT_GPT_RESTART_MODE_t mode)
{
	ALT_STATUS_CODE	result_code;

	switch(mode)
	{
	case ALT_GPT_RESTART_MODE_PERIODIC:
		printf("Timer(tmr_id=%d) response mode set to ALT_GPT_RESTART_MODE_PERIODIC\n", (int)tmr_id);
		break;
	default:	/* case ALT_GPT_RESTART_MODE_ONESHOT: */
		printf("Timer(tmr_id=%d) response mode set to ALT_GPT_RESTART_MODE_ONESHOT\n", (int)tmr_id);
		break;
	}
	result_code = alt_gpt_mode_set(tmr_id, mode);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_mode_set(); result=%d\n", (int)result_code);
	}

	/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_reset(ALT_GPT_TIMER_t tmr_id);	*/
	printf("-- HWLib test : alt_gpt_tmr_reset() --\n");
	result_code = alt_gpt_tmr_reset(tmr_id);
	if(result_code != ALT_E_SUCCESS){
		printf("ERROR!!: alt_gpt_tmr_reset(); result=%d\n", (int)result_code);
	} else {
		printf("SUCCESS!!\n");
	}
	printf("\n");

	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_print_usage(void) 
 * @brief  Display usage of the sample program. 
 * 
 * @details  Following information displays on the console.
 *            - Overview of HPS for push button functions (or operations)
 *            - Overview of HPS for DIP switch functions (or operations)
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2014/08/18  ---  #xxxxx  VK쐬. 
 **//*******************************************************************************/
void sample_test_print_usage(void)
{
	printf("+--- Usage ( Function of HPS-PUSHSW and HPS-DIPSW )  -----------+\n");
	printf("+ PUSHSW HPS 0(SW8) .... Exit Test loop!!!\n");
	printf("+ PUSHSW HPS 1(SW10) ... Display the timer counters and each timeout time(msec).\n");
	printf("+ PUSHSW HPS 2(SW7) .... Display the informations of each timers.(mode,enable,interrupts..)\n");
	printf("+ PUSHSW HPS 3(SW9) .... Reset all general-purpose timer counters.\n");
	printf("+ DIPSW HPS(SW14) 1-4 .. Select timer mode. {Periodic(ON) / One-Shot(OFF)}\n");
	printf("   Each switch is as follows:\n");
	printf("   [1]:OSC1 timer0, [2]:OSC1 timer1, [3]:SP timer0, [4]SP timer1\n");
	printf("+---------------------------------------------------------------+\n\n");
	
#if 0
	@[vňȉ̏sB
	@ - HPSpDIPSW1-4̕ύX_@ŁAeGPT̓샂[hύX܂B
	@@ ܂A[hύXsꍇ̓^C}ċN܂B
	@@ eDIPSW̑Ή͉L̒ʂłB
	@@EDIPSW1 .. OSC1 timer 0 
	@@EDIPSW2 .. OSC1 timer 1 
	@@EDIPSW3 .. SP timer 0 
	@@EDIPSW4 .. SP timer 1 
	@@ieDIPSẂAONȂPERIODIC(User-defined count mode)A
	@@@OFFȂONESHOT(Free-running mode)ɂȂ܂Bj

	@ - HPSpPUSHSW1̉_@ 4GPT̃JE^lу^CAEg܂ł
	@@ cԁi~bj\܂B

	@ - HPSpPUSHSW2̉_@ 4GPT̓i[hA^C}Cl[uA
	@@ 荞݃Cl[uA荞݃yfBOj\܂B

	@ - HPSpPUSHSW3̉_@ 4GPTSĂċNiZbgj܂B

	@ - HPSpPUSHSW0̉_@Ŗ[vI܂B
	@@ (eXgvOI)
#endif
	
	return;
}

/********************************************************************************//*!
 * @fn  void sample_test_main(void) 
 * @brief  Main routine of the sample program. 
 * 
 * @details  Infinite loop structure works. Make the detection of various types 
 *           of switches in the loop, it performs in various testing processes.
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
void sample_test_main(void)
{
	uint32_t ui32portc = 0;
	uint32_t ui32portcbk = 0;
	uint32_t ui32portc_onchg = 0;
	uint32_t ui32dipsw = 0;

	// GPT Timer Restart
	sample_gptimer_reset_all();
	sample_gptimer_print_all_counter();

	printf("==== Start While(1) loop process!!! (Exit PUSHSW0(SW8) becomes ON.) ====\n");
	sample_test_print_usage();
	while(1)
	{
		// ---- Check the DIPSW and PUSHSW. ----
		ui32portc = alt_gpio_port_data_read(ALT_GPIO_PORTC, ALT_GPIO_PORTBIT_SWALL);
		ui32portc = ~ui32portc & ALT_GPIO_PORTBIT_SWALL;	// bit inversion and mask ({ON=0,OFF=1}->{ON=1,OFF=0})
		ui32portc_onchg |= (ui32portc ^ ui32portcbk) & ui32portc;

		// DIPSW HPS 1-4
		if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPSWALL)
		{
			// Detect Time Measurement Index from DIPSW1-4 Value.
			ui32dipsw = (uint32_t)((ui32portc & ALT_GPIO_PORTBIT_DIPSWALL) >> ALT_GPIO_LSBBIT_DIPSWS);
			ui32dipsw *= 1;		// To avoid warnings.

			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			// TImer Mode Change & Restart!!
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS1){
				(void)alt_gpt_tmr_stop(ALT_GPT_OSC1_TMR0);
				if(ui32portc & ALT_GPIO_PORTBIT_DIPS1){
					sample_gptimer_changemode_and_reset(ALT_GPT_OSC1_TMR0, ALT_GPT_RESTART_MODE_PERIODIC);
				} else {
					sample_gptimer_changemode_and_reset(ALT_GPT_OSC1_TMR0, ALT_GPT_RESTART_MODE_ONESHOT);
				}
				(void)alt_gpt_tmr_reset(ALT_GPT_OSC1_TMR0);
				sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR0);
				sample_gptimer_print_specific_settings_status(ALT_GPT_OSC1_TMR0);
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS2){
				(void)alt_gpt_tmr_stop(ALT_GPT_OSC1_TMR1);
				if(ui32portc & ALT_GPIO_PORTBIT_DIPS2){
					sample_gptimer_changemode_and_reset(ALT_GPT_OSC1_TMR1, ALT_GPT_RESTART_MODE_PERIODIC);
				} else {
					sample_gptimer_changemode_and_reset(ALT_GPT_OSC1_TMR1, ALT_GPT_RESTART_MODE_ONESHOT);
				}
				(void)alt_gpt_tmr_reset(ALT_GPT_OSC1_TMR1);
				sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR1);
				sample_gptimer_print_specific_settings_status(ALT_GPT_OSC1_TMR1);
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS3){
				(void)alt_gpt_tmr_stop(ALT_GPT_SP_TMR0);
				if(ui32portc & ALT_GPIO_PORTBIT_DIPS3){
					sample_gptimer_changemode_and_reset(ALT_GPT_SP_TMR0, ALT_GPT_RESTART_MODE_PERIODIC);
				} else {
					sample_gptimer_changemode_and_reset(ALT_GPT_SP_TMR0, ALT_GPT_RESTART_MODE_ONESHOT);
				}
				(void)alt_gpt_tmr_reset(ALT_GPT_SP_TMR0);
				sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR0);
				sample_gptimer_print_specific_settings_status(ALT_GPT_SP_TMR0);
			}
			if((ui32portcbk ^ ui32portc) & ALT_GPIO_PORTBIT_DIPS4){
				(void)alt_gpt_tmr_stop(ALT_GPT_SP_TMR1);
				if(ui32portc & ALT_GPIO_PORTBIT_DIPS4){
					sample_gptimer_changemode_and_reset(ALT_GPT_SP_TMR1, ALT_GPT_RESTART_MODE_PERIODIC);
				} else {
					sample_gptimer_changemode_and_reset(ALT_GPT_SP_TMR1, ALT_GPT_RESTART_MODE_ONESHOT);
				}
				(void)alt_gpt_tmr_reset(ALT_GPT_SP_TMR1);
				sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR1);
				sample_gptimer_print_specific_settings_status(ALT_GPT_SP_TMR1);
			}
		}

		// PUSHSW HPS 0
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH0)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH0;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			// PGT Timer Interrupt Disable
			sample_gptimer_idisable_all();

			printf("======== Print Last GPTimer Settings ========\n");
			sample_gptimer_print_all_settings2();

			// PGT Timer Stop
			sample_gptimer_stop_all();

			break;	// Exit PUSHSW0(SW8) becomes ON.
		}
		// PUSHSW HPS 1
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH1)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH1;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			// Print OSC1_TMR0 and OSC1_TMR1 counter!!
			// Print SP_TMR0 and SP_TMR1 counter!!
			printf("<< ALT_GPT_OSC1_TMR0 >>\n");
			sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR0);
			printf("<< ALT_GPT_OSC1_TMR1 >>\n");
			sample_gptimer_print_specific_counter(ALT_GPT_OSC1_TMR1);
			printf("<< ALT_GPT_SP_TMR0 >>\n");
			sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR0);
			printf("<< ALT_GPT_SP_TMR1 >>\n");
			sample_gptimer_print_specific_counter(ALT_GPT_SP_TMR1);
			printf("\n");
			
		}
		// PUSHSW HPS 2
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH2)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH2;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			sample_gptimer_print_all_settings3();
			
		}
		// PUSHSW HPS 3
		if(ui32portc_onchg & ALT_GPIO_PORTBIT_PUSH3)
		{
			ui32portc_onchg &= ~ALT_GPIO_PORTBIT_PUSH3;
			
			/* ######################################## */
			/* ## Implement the test process here!!! ## */
			/* ######################################## */
			sample_gptimer_reset_all();
			sample_gptimer_print_all_counter2();
			sample_gptimer_print_all_settings3();
		}

		ui32portcbk = ui32portc;
	}
	printf("==== End While(1) loop process. (Exit PUSHSW0(SW8) becomes ON.) ====\n");


	return;
}

// All APIs
/***********************************************************************/
/* Usage:  ALT_STATUS_CODE alt_gpt_all_tmr_uninit(void);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_all_tmr_init(void);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_stop(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_start(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_is_running(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_tmr_reset(ALT_GPT_TIMER_t tmr_id);	*/

/* Usage:  ALT_STATUS_CODE alt_gpt_counter_set(ALT_GPT_TIMER_t tmr_id, uint32_t val);	*/
/* Usage:  uint32_t alt_gpt_counter_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_reset_value_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_maxcounter_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_prescaler_set(ALT_GPT_TIMER_t tmr_id, uint32_t val);	*/
/* Usage:  uint32_t alt_gpt_prescaler_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_freq_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_time_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_time_millisecs_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_time_microsecs_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_curtime_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_curtime_millisecs_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_curtime_microsecs_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_curtime_nanosecs_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_maxtime_get(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  uint32_t alt_gpt_maxtime_millisecs_get(ALT_GPT_TIMER_t tmr_id);	*/

/* Usage:  ALT_STATUS_CODE alt_gpt_int_disable(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_int_enable(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  bool alt_gpt_int_is_enabled(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_int_clear_pending(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_int_is_pending(ALT_GPT_TIMER_t tmr_id);	*/
/* Usage:  ALT_STATUS_CODE alt_gpt_int_if_pending_clear(ALT_GPT_TIMER_t tmr_id);	*/

/* Usage:  ALT_STATUS_CODE alt_gpt_mode_set(ALT_GPT_TIMER_t tmr_id, ALT_GPT_RESTART_MODE_t mode);	*/
/* Usage:  int32_t alt_gpt_mode_get(ALT_GPT_TIMER_t tmr_id);	*/
/***********************************************************************/

////////////////////////////// Common Functions //////////////////////////////////////// 
/********************************************************************************//*!
 * @fn  static void sample_gpio_config_helioboard(void) 
 * @brief  Configure the Helio Board GPIO settings. 
 * 
 * @details  Performs the following processing. 
 *           - Gets the contents of the current port settings to gpio_config_array. 
 *           - Global timer counter value of the measurement at the start
 *           - Setting DIPSW and PUSHSW via bit index configuration func. 
 * @param[in,out] void  none 
 * @retval void  none 
 * 
 * @attention  nothing. 
 * @pre        nothing. 
 * @post       nothing. 
 * @note       nothing. 
 * 
 * @date <b> History: \<Date\> \<Rev\> \<Keyword\> \<Details\> </b>
 * @date 2013/11/28  ---  #6334  VK쐬. 
 * @date 2014/01/29  ---  -----  ُ̍CiRgA֐̃xړύX). 
 **//*******************************************************************************/
static void sample_gpio_config_helioboard(void)
{
	ALT_STATUS_CODE result_code;
	ALT_GPIO_CONFIG_RECORD_t gpio_config_array[10];
	ALT_GPIO_CONFIG_RECORD_t* p_gpiocfg;

	// Setting OUT-PORTiGPIO61j via normal configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_port_config(	ALT_GPIO_PORT_t gpio_pid,
	 * 												uint32_t mask,
	 * 												ALT_GPIO_PIN_DIR_t dir,
	 * 												ALT_GPIO_PIN_TYPE_t type,
	 * 												ALT_GPIO_PIN_POL_t pol,
	 * 												ALT_GPIO_PIN_DEBOUNCE_t debounc,
	 * 												uint32_t data); 						*/
	/****************************************************************************************/
	result_code = alt_gpio_port_config(ALT_GPIO_PORTC, ALT_GPIO_BIT3,
			ALT_GPIO_PIN_OUTPUT, ALT_GPIO_PIN_LEVEL_TRIG_INT, ALT_GPIO_PIN_ACTIVE_HIGH, ALT_GPIO_PIN_NODEBOUNCE,
			ALT_GPIO_BIT3);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_port_config() result=%d\n", (int)result_code);
	}

	// Gets the contents of the current port settings to gpio_config_array.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config_get(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len);	*/
	/****************************************************************************************/
	gpio_config_array[0].signal_number = ALT_GPIO_1BIT_61;
	gpio_config_array[1].signal_number = ALT_HLGPI_6;
	gpio_config_array[2].signal_number = ALT_HLGPI_7;
	gpio_config_array[3].signal_number = ALT_HLGPI_8;
	gpio_config_array[4].signal_number = ALT_HLGPI_9;
	gpio_config_array[5].signal_number = ALT_HLGPI_10;
	gpio_config_array[6].signal_number = ALT_HLGPI_11;
	gpio_config_array[7].signal_number = ALT_HLGPI_12;
	gpio_config_array[8].signal_number = ALT_HLGPI_13;
	gpio_config_array[9].signal_number = ALT_END_OF_GPIO_SIGNALS;
	result_code = alt_gpio_group_config_get(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config_get() result=%d\n", (int)result_code);
	}

	// Setting DIPSW and PUSHSW via bit index configuration func.
	/****************************************************************************************/
	/* Usage: ALT_STATUS_CODE alt_gpio_group_config(ALT_GPIO_CONFIG_RECORD_t* config_array, uint32_t len); */
	/****************************************************************************************/
	for(int i=1; i<10; i++)
	{
		p_gpiocfg = &gpio_config_array[i];
		p_gpiocfg->direction = ALT_GPIO_PIN_INPUT;
		p_gpiocfg->type = ALT_GPIO_PIN_LEVEL_TRIG_INT;
		p_gpiocfg->polarity = ALT_GPIO_PIN_ACTIVE_HIGH;
		p_gpiocfg->debounce = ALT_GPIO_PIN_NODEBOUNCE;
		p_gpiocfg->data = ALT_GPIO_PIN_DATAZERO;
	}
	result_code = alt_gpio_group_config(gpio_config_array, 10);
	if(result_code != ALT_E_SUCCESS)
	{
		printf("ERROR!!: alt_gpio_group_config() result=%d\n", (int)result_code);
	}

	return;
}

/***********************************************************************************
 * end of file 
 ***********************************************************************************/
